@@ -39,6 +39,8 @@ class CloudInfo:
         """Protocol to connect to the cloud provider"""
         self.path = ""
         """Path to connect to the cloud provider"""
+        self.extra = {}
+        """Extra fields needed to represent the cloud provider (e.g. tenant, domain ...)"""
 
     def getCloudConnector(self, inf):
         """
@@ -67,6 +69,27 @@ class CloudInfo:
 
         return res
 
+    @staticmethod
+    def add_extra_fields(auth, cloud_item):
+        # Add extra fields in case of OpenStack and EGI sites
+        if 'type' in auth and auth['type'] == "OpenStack":
+            # This should be the same as OpenStackCloudConnector.DEFAULT_AUTH_VERSION
+            auth_version = auth['auth_version'] if 'auth_version' in auth else "2.0_password"
+            cloud_item.extra['auth_version'] = auth_version
+            if auth_version == "3.x_oidc_access_token":
+                # in this case username represents the identity provider
+                if 'username' in auth and auth['username']:
+                    cloud_item.extra['username'] = auth['username']
+                # and the domain the project/tenant name
+                if 'domain' in auth and auth['domain']:
+                    cloud_item.extra['domain'] = auth['domain']
+            elif "password" in auth_version:
+                if 'tenant' in auth and auth['tenant']:
+                    cloud_item.extra['tenant'] = auth['tenant']
+        elif 'type' in auth and auth['type'] == "EGI":
+            if 'vo' in auth and auth['vo']:
+                cloud_item.extra["vo"] = auth['vo']
+
     @staticmethod
     def get_cloud_list(auth_data):
         """
@@ -75,7 +98,7 @@ class CloudInfo:
         res = []
 
         for i, auth in enumerate(auth_data.auth_list):
-            if 'type' in auth and auth['type'] not in ['InfrastructureManager', 'VMRC']:
+            if 'type' in auth and auth['type'] not in ['InfrastructureManager', 'VMRC', 'AppDBIS']:
                 cloud_item = CloudInfo()
                 cloud_item.type = auth['type']
                 if 'id' in auth.keys() and auth['id']:
@@ -109,6 +132,9 @@ class CloudInfo:
                 except Exception:
                     pass
 
+                # Add extra fields in case of OpenStack and EGI sites
+                CloudInfo.add_extra_fields(auth, cloud_item)
+
                 res.append(cloud_item)
 
         return res
@@ -122,3 +148,19 @@ class CloudInfo:
         nwecloud = CloudInfo()
         nwecloud.__dict__.update(dic)
         return nwecloud
+
+    def get_port(self):
+        protocol = self.protocol or "http"
+        port = self.port
+        if port == -1:
+            if protocol == "http":
+                port = 80
+            elif protocol == "https":
+                port = 443
+        return port
+
+    def get_url(self):
+        protocol = self.protocol or "http"
+        if not protocol:
+            protocol = "http"
+        return protocol + "://" + self.server + ":" + str(self.get_port()) + self.path
