@@ -234,7 +234,7 @@ class InfrastructureInfo:
         Stop all the Ctxt threads
         """
         # Stop the Ctxt thread if it is alive.
-        if self.cm and self.cm.isAlive():
+        if self.cm and self.cm.is_alive():
             self.cm.stop()
 
         # kill all the ctxt processes in the VMs
@@ -376,7 +376,7 @@ class InfrastructureInfo:
                 self.radl.deploys = radl.deploys
 
             # Associate private networks with cloud providers
-            for d, _, _ in deployed_vms:
+            for d in deployed_vms:
                 for private_net in [net.id for net in radl.networks if not net.isPublic() and
                                     net.id in radl.get_system_by_name(d.id).getNetworkIDs()]:
                     if private_net in self.private_networks:
@@ -479,7 +479,8 @@ class InfrastructureInfo:
         max_vms_connected = -1
         for vm in self.get_vm_list():
             vms_connected = -1
-            if vm.getOS() and vm.getOS().lower() == 'linux' and (vm.hasPublicNet() or vm.getProxyHost()):
+            if (vm.contextualize() and vm.getOS() and vm.getOS().lower() == 'linux' and
+                    (vm.hasPublicNet() or vm.getProxyHost())):
                 # check that is connected with other VMs
                 vms_connected = 0
                 for other_vm in self.get_vm_list():
@@ -536,12 +537,12 @@ class InfrastructureInfo:
                 self.ctxt_tasks.put(elem)
 
     def get_ctxt_process_names(self):
-        return [t.name for t in self.conf_threads if t.isAlive()]
+        return [t.name for t in self.conf_threads if t.is_alive()]
 
     def is_ctxt_process_running(self):
         all_finished = True
         for t in self.conf_threads:
-            if t.isAlive():
+            if t.is_alive():
                 all_finished = False
         if all_finished:
             self.conf_threads = []
@@ -577,12 +578,18 @@ class InfrastructureInfo:
             max_ctxt_time = Config.MAX_CONTEXTUALIZATION_TIME
 
         self.configured = None
+        all_configure_disabled = True
         for vm in self.get_vm_list():
             # Assure to update the VM status before running the ctxt process
             vm.update_status(auth)
             vm.cont_out = ""
             vm.cloud_connector = None
             vm.configured = None
+            if vm.contextualize():
+                all_configure_disabled = False
+
+        if all_configure_disabled:
+            ctxt = False
 
         if not ctxt:
             InfrastructureInfo.logger.info("Inf ID: " + str(self.id) + ": Contextualization disabled by the RADL. " +
@@ -612,6 +619,10 @@ class InfrastructureInfo:
 
             use_dist = len(self.get_vm_list()) > Config.VM_NUM_USE_CTXT_DIST
             for cont, vm in enumerate(self.get_vm_list()):
+                if not vm.contextualize():
+                    vm.configured = True
+                    vm.cont_out = "Contextualization disabled"
+                    continue
                 tasks = {}
 
                 # Add basic tasks for all VMs
@@ -656,7 +667,7 @@ class InfrastructureInfo:
 
         self.add_ctxt_tasks(ctxt_task)
 
-        if self.cm is None or not self.cm.isAlive():
+        if self.cm is None or not self.cm.is_alive():
             self.cm = IM.ConfManager.ConfManager(self, auth, max_ctxt_time)
             self.cm.start()
         else:
@@ -668,49 +679,58 @@ class InfrastructureInfo:
             # restart the failed step
             self.cm.failed_step = []
 
+    def _is_authorized(self, self_im_auth, auth):
+        """
+        Checks if the auth data provided is authorized to access this infrastructure
+        """
+        res = False
+        for other_im_auth in auth.getAuthInfo("InfrastructureManager"):
+            res = True
+            for elem in ['username', 'password']:
+                if elem not in other_im_auth:
+                    res = False
+                    break
+                if elem not in self_im_auth:
+                    InfrastructureInfo.logger.error("Inf ID %s has not elem %s in the auth data" % (self.id, elem))
+                if self_im_auth[elem] != other_im_auth[elem]:
+                    res = False
+                    break
+
+            if 'token' in self_im_auth:
+                if 'token' not in other_im_auth:
+                    res = False
+                    break
+                decoded_token = JWT().get_info(other_im_auth['token'])
+                issuer = str(decoded_token['iss'])
+                if not issuer.endswith('/'):
+                    issuer += '/'
+                password = issuer + str(decoded_token['sub'])
+                # check that the token provided is associated with the current owner of the inf.
+                if self_im_auth['password'] != password:
+                    res = False
+                    break
+
+                # In case of OIDC token update it in each call to get a fresh version
+                self_im_auth['token'] = other_im_auth['token']
+
+            if (self_im_auth['username'].startswith(InfrastructureInfo.OPENID_USER_PREFIX) and
+                    'token' not in other_im_auth):
+                # This is a OpenID user do not enable to get data using user/pass creds
+                InfrastructureInfo.logger.warn("Inf ID %s: A non OpenID user tried to access it." % self.id)
+                res = False
+                break
+
+            if res:
+                return res
+
     def is_authorized(self, auth):
         """
         Checks if the auth data provided is authorized to access this infrastructure
         """
         if self.auth is not None:
-            self_im_auth = self.auth.getAuthInfo("InfrastructureManager")[0]
-
-            for other_im_auth in auth.getAuthInfo("InfrastructureManager"):
-
-                res = True
-                for elem in ['username', 'password']:
-                    if elem not in other_im_auth:
-                        res = False
-                        break
-                    if elem not in self_im_auth:
-                        InfrastructureInfo.logger.error("Inf ID %s has not elem %s in the auth data" % (self.id, elem))
-                    if self_im_auth[elem] != other_im_auth[elem]:
-                        res = False
-                        break
-
-                if 'token' in self_im_auth:
-                    if 'token' not in other_im_auth:
-                        res = False
-                        break
-                    decoded_token = JWT().get_info(other_im_auth['token'])
-                    password = str(decoded_token['iss']) + str(decoded_token['sub'])
-                    # check that the token provided is associated with the current owner of the inf.
-                    if self_im_auth['password'] != password:
-                        res = False
-                        break
-
-                    # In case of OIDC token update it in each call to get a fresh version
-                    self_im_auth['token'] = other_im_auth['token']
-
-                if (self_im_auth['username'].startswith(InfrastructureInfo.OPENID_USER_PREFIX) and
-                        'token' not in other_im_auth):
-                    # This is a OpenID user do not enable to get data using user/pass creds
-                    InfrastructureInfo.logger.warn("Inf ID %s: A non OpenID user tried to access it." % self.id)
-                    res = False
-                    break
-
-                if res:
-                    return res
+            for self_im_auth in self.auth.getAuthInfo("InfrastructureManager"):
+                if self._is_authorized(self_im_auth, auth):
+                    return True
 
             return False
         else:
@@ -751,13 +771,13 @@ class InfrastructureInfo:
                 raise Exception("Infrastructure deleted. Do not add resources.")
             self.adding = value
 
-    def get_auth(self):
+    def get_auth(self, only_user_pass=False):
         """
         Return IM auth in a simple way
         """
         if self.auth is not None:
             self_im_auth = self.auth.getAuthInfo("InfrastructureManager")[0]
-            if 'token' in self_im_auth:
+            if 'token' in self_im_auth and not only_user_pass:
                 return self_im_auth['token']
             elif 'username' in self_im_auth and 'password' in self_im_auth:
                 return "%s:%s" % (self_im_auth['username'], self_im_auth['password'])
@@ -766,3 +786,15 @@ class InfrastructureInfo:
                 return None
         else:
             return None
+
+    def change_auth(self, new_auth, overwrite):
+        """Changes the auth data of the infrastructure."""
+        if new_auth is not None:
+            im_auth = new_auth.getAuthInfo("InfrastructureManager")
+            if im_auth:
+                if 'username' in im_auth[0]:
+                    InfrastructureInfo.logger.debug("Changing user to: %s" % im_auth[0]['username'])
+                if overwrite:
+                    self.auth = Authentication(im_auth)
+                else:
+                    self.auth.auth_list.extend(im_auth)
