@@ -27,6 +27,7 @@ import IM.InfrastructureInfo
 import IM.InfrastructureList
 
 from IM.VMRC import VMRC
+from IM.AppDBIS import AppDBIS
 from IM.CloudInfo import CloudInfo
 from IM.auth import Authentication
 from IM.recipe import Recipe
@@ -158,9 +159,11 @@ class InfrastructureManager:
         return deploy_groups
 
     @staticmethod
-    def _launch_deploy(sel_inf, deploy, cloud_id, cloud, concrete_systems, radl, auth, deployed_vm):
+    def _launch_deploy(sel_inf, dep, cloud_id, cloud, concrete_systems, radl, auth, deployed_vm):
         """Launch a deploy."""
 
+        # Clone the deploy to avoid changes in the original inf deploys
+        deploy = dep.clone()
         if deploy.vm_number <= 0:
             InfrastructureManager.logger.warning(
                 "Inf ID: %s: deploy %s with 0 num: Ignoring." % (sel_inf.id, deploy.id))
@@ -278,10 +281,11 @@ class InfrastructureManager:
         # Add or update configures
         for s in radl.configures:
             # first check that the YAML is correct
-            try:
-                yaml.safe_load(s.recipes)
-            except Exception as ex:
-                raise Exception("Error parsing YAML: %s" % str(ex))
+            if s.recipes:
+                try:
+                    yaml.safe_load(s.recipes)
+                except Exception as ex:
+                    raise Exception("Error parsing YAML: %s" % str(ex))
             sel_inf.radl.add(s.clone(), "replace")
             InfrastructureManager.logger.info(
                 "Inf ID: " + sel_inf.id + ": " +
@@ -383,9 +387,10 @@ class InfrastructureManager:
         return concrete_system, score
 
     @staticmethod
-    def systems_with_vmrc(sel_inf, radl, auth):
+    def systems_with_iis(sel_inf, radl, auth):
         """
-        Concrete systems using VMRC
+        Concrete systems using Image Information Systems.
+        Currently supported VMRC and AppDBIS
         NOTE: consider not-fake deploys (vm_number > 0)
         """
         # Get VMRC credentials
@@ -394,12 +399,20 @@ class InfrastructureManager:
             if 'host' in vmrc_elem and 'username' in vmrc_elem and 'password' in vmrc_elem:
                 vmrc_list.append(VMRC(vmrc_elem['host'], vmrc_elem['username'], vmrc_elem['password']))
 
+        # Get AppDBIS credentials
+        appdbis_list = []
+        for appdbis_elem in auth.getAuthInfo('AppDBIS'):
+            host = None
+            if 'host' in appdbis_elem:
+                host = appdbis_elem['host']
+            appdbis_list.append(AppDBIS(host))
+
         systems_with_vmrc = {}
         for system_id in set([d.id for d in radl.deploys if d.vm_number > 0]):
             s = radl.get_system_by_name(system_id)
 
-            if not s.getValue("disk.0.image.url") and len(vmrc_list) == 0:
-                raise Exception("No correct VMRC auth data provided nor image URL")
+            if not s.getValue("disk.0.image.url") and len(vmrc_list + appdbis_list) == 0:
+                raise Exception("No correct VMRC or AppDBIS auth data provided nor image URL")
 
             if Config.SINGLE_SITE:
                 image_id = os.path.basename(s.getValue("disk.0.image.url"))
@@ -421,13 +434,14 @@ class InfrastructureManager:
                     s_without_apps.addFeature(f)
 
             vmrc_res = [s0 for vmrc in vmrc_list for s0 in vmrc.search_vm(s)]
+            appdbis_res = [s0 for appdbis in appdbis_list for s0 in appdbis.search_vm(s)]
             # Check that now the image URL is in the RADL
-            if not s.getValue("disk.0.image.url") and not vmrc_res:
-                sel_inf.add_cont_msg("No VMI obtained from VMRC to system: " + system_id)
-                raise Exception("No VMI obtained from VMRC to system: " + system_id)
+            if not s.getValue("disk.0.image.url") and not vmrc_res and not appdbis_res:
+                sel_inf.add_cont_msg("No VMI obtained from VMRC nor AppDBIS to system: " + system_id)
+                raise Exception("No VMI obtained from VMRC nor AppDBIS to system: " + system_id)
 
             n = [s_without_apps.clone().applyFeatures(s0, conflict="other", missing="other")
-                 for s0 in vmrc_res]
+                 for s0 in (vmrc_res + appdbis_res)]
             systems_with_vmrc[system_id] = n if n else [s_without_apps]
 
         return systems_with_vmrc
@@ -552,7 +566,7 @@ class InfrastructureManager:
 
         # Concrete systems using VMRC
         try:
-            systems_with_vmrc = InfrastructureManager.systems_with_vmrc(sel_inf, radl, auth)
+            systems_with_iis = InfrastructureManager.systems_with_iis(sel_inf, radl, auth)
         except Exception as ex:
             sel_inf.configured = False
             sel_inf.add_cont_msg("Error getting VM images: %s" % str(ex))
@@ -564,7 +578,7 @@ class InfrastructureManager:
         cloud_list = dict([(c.id, c.getCloudConnector(sel_inf)) for c in CloudInfo.get_cloud_list(auth)])
         concrete_systems = {}
         for cloud_id, cloud in cloud_list.items():
-            for system_id, systems in systems_with_vmrc.items():
+            for system_id, systems in systems_with_iis.items():
                 s1 = [InfrastructureManager._compute_score(s.clone().applyFeatures(s0,
                                                                                    conflict="other",
                                                                                    missing="other").concrete(),
@@ -592,6 +606,7 @@ class InfrastructureManager:
 
         # Launch every group in the same cloud provider
         deployed_vm = {}
+        deploy_items = []
         for deploy_group in deploy_groups:
             if not deploy_group:
                 InfrastructureManager.logger.warning("Inf ID: %s: No VMs to deploy!" % sel_inf.id)
@@ -602,19 +617,24 @@ class InfrastructureManager:
 
             cloud_id = deploys_group_cloud[id(deploy_group)]
             cloud = cloud_list[cloud_id]
-            if Config.MAX_SIMULTANEOUS_LAUNCHES > 1:
-                pool = ThreadPool(processes=Config.MAX_SIMULTANEOUS_LAUNCHES)
-                pool.map(
-                    lambda deploy: InfrastructureManager._launch_deploy(sel_inf, deploy, cloud_id,
-                                                                        cloud, concrete_systems, radl, auth,
-                                                                        deployed_vm),
-                    deploy_group)
-                pool.close()
-            else:
-                for deploy in deploy_group:
-                    InfrastructureManager._launch_deploy(sel_inf, deploy, cloud_id,
-                                                         cloud, concrete_systems, radl,
-                                                         auth, deployed_vm)
+
+            for d in deploy_group:
+                deploy_items.append((d, cloud_id, cloud))
+
+        # Now launch all the deployments
+        if Config.MAX_SIMULTANEOUS_LAUNCHES > 1:
+            pool = ThreadPool(processes=Config.MAX_SIMULTANEOUS_LAUNCHES)
+            pool.map(
+                lambda depitem: InfrastructureManager._launch_deploy(sel_inf, depitem[0], depitem[1],
+                                                                     depitem[2], concrete_systems, radl, auth,
+                                                                     deployed_vm),
+                deploy_items)
+            pool.close()
+        else:
+            for deploy, cloud_id, cloud in deploy_items:
+                InfrastructureManager._launch_deploy(sel_inf, deploy, cloud_id,
+                                                     cloud, concrete_systems, radl,
+                                                     auth, deployed_vm)
 
         # We make this to maintain the order of the VMs in the sel_inf.vm_list
         # according to the deploys shown in the RADL
@@ -648,9 +668,7 @@ class InfrastructureManager:
 
         error_msg = ""
         # Add the new virtual machines to the infrastructure
-        sel_inf.update_radl(radl,
-                            [(d, deployed_vm[d], concrete_systems[d.cloud_id][d.id][0]) for d in deployed_vm],
-                            False)
+        sel_inf.update_radl(radl, deployed_vm, False)
         if all_failed:
             InfrastructureManager.logger.error("VMs failed when adding to Inf ID: %s" % sel_inf.id)
             sel_inf.add_cont_msg("All VMs failed. No contextualize.")
@@ -1278,7 +1296,7 @@ class InfrastructureManager:
                     found = False
                     user_db = json.load(open(Config.USER_DB, "r"))
                     for user in user_db['users']:
-                        if user['username'] == auth[0]['username'] and user['password'] == auth[0]['password']:
+                        if user['username'] == auth['username'] and user['password'] == auth['password']:
                             found = True
                             break
                     return found
@@ -1325,6 +1343,7 @@ class InfrastructureManager:
                 raise InvaliddUserException("Invalid InfrastructureManager credentials. Audience not accepted.")
 
         if Config.OIDC_SCOPES and Config.OIDC_CLIENT_ID and Config.OIDC_CLIENT_SECRET:
+            OpenIDClient.INSTROSPECT_PATH = Config.OIDC_INSTROSPECT_PATH
             success, res = OpenIDClient.get_token_introspection(token,
                                                                 Config.OIDC_CLIENT_ID,
                                                                 Config.OIDC_CLIENT_SECRET,
@@ -1351,6 +1370,7 @@ class InfrastructureManager:
 
         try:
             # Now try to get user info
+            OpenIDClient.USER_INFO_PATH = Config.OIDC_USER_INFO_PATH
             success, userinfo = OpenIDClient.get_user_info_request(token, Config.VERIFI_SSL)
             if success:
                 # convert to username to use it in the rest of the IM
@@ -1361,7 +1381,10 @@ class InfrastructureManager:
                     im_auth['username'] += str(userinfo.get("name"))
                 else:
                     im_auth['username'] += str(userinfo.get("sub"))
-                im_auth['password'] = str(decoded_token['iss']) + str(userinfo.get("sub"))
+                issuer = str(decoded_token['iss'])
+                if not issuer.endswith('/'):
+                    issuer += '/'
+                im_auth['password'] = issuer + str(userinfo.get("sub"))
         except Exception as ex:
             InfrastructureManager.logger.exception("Error trying to validate OIDC auth token: %s" % str(ex))
             raise Exception("Error trying to validate OIDC auth token: %s" % str(ex))
@@ -1399,31 +1422,55 @@ class InfrastructureManager:
         else:
             return auth
 
+    @staticmethod
+    def gen_auth_from_appdb(auth):
+        # Gen EGI auth for all the sites that supports the specified VO
+        appdbis_auth = auth.getAuthInfo("AppDBIS")
+        if appdbis_auth and "vo" in appdbis_auth[0] and "token" in appdbis_auth[0]:
+            vo = appdbis_auth[0]["vo"]
+            # To avoid connecting with AppDBIS again
+            del appdbis_auth[0]["vo"]
+            if "host" in appdbis_auth[0]:
+                appdbis = AppDBIS(appdbis_auth[0]["host"])
+            else:
+                appdbis = AppDBIS()
+            InfrastructureManager.logger.debug("Getting auth data from AppDBIS")
+            code, sites = appdbis.get_sites_supporting_vo(vo)
+            if code == 200:
+                for site_name, site_url, project_id in sites:
+                    auth_site = {"id": site_name, "host": site_url, "type": "OpenStack",
+                                 "username": "egi.eu", "tenant": "openid", "auth_version": "3.x_oidc_access_token",
+                                 "domain": project_id, "password": appdbis_auth[0]["token"], "vo": vo}
+                    auth.auth_list.append(auth_site)
+            else:
+                InfrastructureManager.logger.error("Error getting auth data from AppDBIS: %s" % sites)
+        return auth
+
     @staticmethod
     def check_auth_data(auth):
         # First check if it is configured to check the users from a list
-        auth = InfrastructureManager.get_auth_from_vault(auth)
         im_auth = auth.getAuthInfo("InfrastructureManager")
 
         if not im_auth:
             raise InvaliddUserException("No credentials provided for the InfrastructureManager.")
 
-        if Config.FORCE_OIDC_AUTH and "token" not in im_auth[0]:
-            raise InvaliddUserException("No token provided for the InfrastructureManager.")
+        for im_auth_item in im_auth:
+            if Config.FORCE_OIDC_AUTH and "token" not in im_auth_item:
+                raise InvaliddUserException("No token provided for the InfrastructureManager.")
 
-        # First check if an OIDC token is included
-        if "token" in im_auth[0]:
-            InfrastructureManager.check_oidc_token(im_auth[0])
-        elif "username" in im_auth[0]:
-            if im_auth[0]['username'].startswith(IM.InfrastructureInfo.InfrastructureInfo.OPENID_USER_PREFIX):
-                # This is a OpenID user do not enable to get data using user/pass creds
-                raise InvaliddUserException("Invalid username used for the InfrastructureManager.")
-        else:
-            raise InvaliddUserException("No username nor token for the InfrastructureManager.")
+            # First check if an OIDC token is included
+            if "token" in im_auth_item:
+                InfrastructureManager.check_oidc_token(im_auth_item)
+            elif "username" in im_auth_item:
+                if im_auth_item['username'].startswith(IM.InfrastructureInfo.InfrastructureInfo.OPENID_USER_PREFIX):
+                    # This is a OpenID user do not enable to get data using user/pass creds
+                    raise InvaliddUserException("Invalid username used for the InfrastructureManager.")
+            else:
+                raise InvaliddUserException("No username nor token for the InfrastructureManager.")
 
-        # Now check if the user is in authorized
-        if not InfrastructureManager.check_im_user(im_auth):
-            raise InvaliddUserException()
+            # Now check if the user is in authorized
+            if not InfrastructureManager.check_im_user(im_auth_item):
+                raise InvaliddUserException()
 
         if Config.SINGLE_SITE:
             vmrc_auth = auth.getAuthInfo("VMRC")
@@ -1438,6 +1485,8 @@ class InfrastructureManager:
             auth = Authentication(auth_list)
 
         # We have to check if TTS is needed for other auth item
+        auth = InfrastructureManager.get_auth_from_vault(auth)
+        auth = InfrastructureManager.gen_auth_from_appdb(auth)
         return auth
 
     @staticmethod
@@ -1642,7 +1691,7 @@ class InfrastructureManager:
 
         Arguments:
           - cloud_id(string): ID of the cloud provider specified in the Authentication data
-          - auth_data(:py:class:`dict` of str objects): Authentication data to access cloud provider.
+          - auth(Authentication): parsed authentication tokens to access cloud provider.
 
         Returns: a CloudConnection object.
         """
@@ -1664,7 +1713,7 @@ class InfrastructureManager:
 
         Arguments:
           - cloud_id(string): ID of the cloud provider specified in the Authentication data
-          - auth_data(:py:class:`dict` of str objects): Authentication data to access cloud provider.
+          - auth(Authentication): parsed authentication tokens to access cloud provider.
           - filters(:py:class:`dict` of str objects): Pair key value to filter the list of images.
                                                      It is cloud provider specific.
 
@@ -1672,7 +1721,15 @@ class InfrastructureManager:
         """
         # First check the auth data
         auth = InfrastructureManager.check_auth_data(auth)
-        return InfrastructureManager._get_cloud_conn(cloud_id, auth).list_images(auth)
+        appdbis_auth = auth.getAuthInfo("AppDBIS")
+        if appdbis_auth and "token" in appdbis_auth[0] and cloud_id == appdbis_auth[0]['id']:
+            if "host" in appdbis_auth[0]:
+                appdbis = AppDBIS(appdbis_auth[0]["host"])
+            else:
+                appdbis = AppDBIS()
+            return appdbis.list_images(filters)
+        else:
+            return InfrastructureManager._get_cloud_conn(cloud_id, auth).list_images(auth, filters)
 
     @staticmethod
     def GetCloudQuotas(cloud_id, auth):
@@ -1681,7 +1738,7 @@ class InfrastructureManager:
 
         Arguments:
           - cloud_id(string): ID of the cloud provider specified in the Authentication data
-          - auth_data(:py:class:`dict` of str objects): Authentication data to access cloud provider.
+          - auth(Authentication): parsed authentication tokens to access cloud provider.
 
         Returns: dict with the following structure (if there are no limit in some metric, value is set to 1):
                     {
@@ -1696,3 +1753,30 @@ class InfrastructureManager:
         # First check the auth data
         auth = InfrastructureManager.check_auth_data(auth)
         return InfrastructureManager._get_cloud_conn(cloud_id, auth).get_quotas(auth)
+
+    @staticmethod
+    def ChangeInfrastructureAuth(inf_id, new_auth, overwrite, auth):
+        """
+        Get the number of used and available resources in the cloud provider
+
+        Arguments:
+          - inf_id(str): infrastructure id.
+          - auth(Authentication): parsed authentication tokens.
+          - new_auth(Authentication): New parsed authentication tokens.
+          - overwrite(bool): Flag to set if the new auth data will overwrite the currrent one or it wil
+                             be appended.
+        """
+        # First check the auth data
+        InfrastructureManager.logger.info("Changing user of the Inf ID: " + str(inf_id))
+        auth = InfrastructureManager.check_auth_data(auth)
+        sel_inf = InfrastructureManager.get_infrastructure(inf_id, auth)
+        try:
+            # Check also the new auth data
+            new_auth = InfrastructureManager.check_auth_data(new_auth)
+        except InvaliddUserException as ex:
+            raise InvaliddUserException("Invalid new infrastructure data provided: %s" % ex)
+        # Only add new user if it is not already authorized or we are overwriting
+        if not sel_inf.is_authorized(new_auth) or overwrite:
+            sel_inf.change_auth(new_auth, overwrite)
+        IM.InfrastructureList.InfrastructureList.save_data(inf_id)
+        return ""
