@@ -560,7 +560,7 @@ def RESTCreateInfrastructure():
     except InvaliddUserException as ex:
         return return_error(401, "Error Getting Inf. info: %s" % get_ex_error(ex))
     except DisabledFunctionException as ex:
-        return return_error(403, "Error Destroying Inf: %s" % get_ex_error(ex))
+        return return_error(403, "Error Getting Inf, info: %s" % get_ex_error(ex))
     except Exception as ex:
         logger.exception("Error Creating Inf.")
         return return_error(400, "Error Creating Inf.: %s" % get_ex_error(ex))
@@ -590,7 +590,7 @@ def RESTImportInfrastructure():
     except InvaliddUserException as ex:
         return return_error(401, "Error Impporting Inf.: %s" % get_ex_error(ex))
     except DisabledFunctionException as ex:
-        return return_error(403, "Error Destroying Inf: %s" % get_ex_error(ex))
+        return return_error(403, "Error Impporting Inf: %s" % get_ex_error(ex))
     except Exception as ex:
         logger.exception("Error Impporting Inf.")
         return return_error(400, "Error Impporting Inf.: %s" % get_ex_error(ex))
@@ -786,7 +786,7 @@ def RESTAddResource(infid=None):
     except UnauthorizedUserException as ex:
         return return_error(403, "Error Adding resources: %s" % get_ex_error(ex))
     except DisabledFunctionException as ex:
-        return return_error(403, "Error Destroying Inf: %s" % get_ex_error(ex))
+        return return_error(403, "Error Adding resources: %s" % get_ex_error(ex))
     except Exception as ex:
         logger.exception("Error Adding resources")
         return return_error(400, "Error Adding resources: %s" % get_ex_error(ex))
@@ -824,7 +824,7 @@ def RESTRemoveResource(infid=None, vmid=None):
     except IncorrectVMException as ex:
         return return_error(404, "Error Removing resources: %s" % get_ex_error(ex))
     except DisabledFunctionException as ex:
-        return return_error(403, "Error Destroying Inf: %s" % get_ex_error(ex))
+        return return_error(403, "Error Removing resources: %s" % get_ex_error(ex))
     except Exception as ex:
         logger.exception("Error Removing resources")
         return return_error(400, "Error Removing resources: %s" % get_ex_error(ex))
@@ -866,7 +866,7 @@ def RESTAlterVM(infid=None, vmid=None):
     except IncorrectVMException as ex:
         return return_error(404, "Error modifying resources: %s" % get_ex_error(ex))
     except DisabledFunctionException as ex:
-        return return_error(403, "Error Destroying Inf: %s" % get_ex_error(ex))
+        return return_error(403, "Error modifying resources: %s" % get_ex_error(ex))
     except Exception as ex:
         logger.exception("Error modifying resources")
         return return_error(400, "Error modifying resources: %s" % get_ex_error(ex))
@@ -910,7 +910,7 @@ def RESTReconfigureInfrastructure(infid=None):
     except UnauthorizedUserException as ex:
         return return_error(403, "Error reconfiguring infrastructure: %s" % get_ex_error(ex))
     except DisabledFunctionException as ex:
-        return return_error(403, "Error Destroying Inf: %s" % get_ex_error(ex))
+        return return_error(403, "Error reconfiguring infrastructure: %s" % get_ex_error(ex))
     except Exception as ex:
         logger.exception("Error reconfiguring infrastructure")
         return return_error(400, "Error reconfiguring infrastructure: %s" % get_ex_error(ex))
@@ -933,7 +933,7 @@ def RESTStartInfrastructure(infid=None):
     except UnauthorizedUserException as ex:
         return return_error(403, "Error starting infrastructure: %s" % get_ex_error(ex))
     except DisabledFunctionException as ex:
-        return return_error(403, "Error Destroying Inf: %s" % get_ex_error(ex))
+        return return_error(403, "Error starting infrastructure: %s" % get_ex_error(ex))
     except Exception as ex:
         logger.exception("Error starting infrastructure")
         return return_error(400, "Error starting infrastructure: %s" % get_ex_error(ex))
@@ -956,7 +956,7 @@ def RESTStopInfrastructure(infid=None):
     except UnauthorizedUserException as ex:
         return return_error(403, "Error stopping infrastructure: %s" % get_ex_error(ex))
     except DisabledFunctionException as ex:
-        return return_error(403, "Error Destroying Inf: %s" % get_ex_error(ex))
+        return return_error(403, "Error stopping infrastructure: %s" % get_ex_error(ex))
     except Exception as ex:
         logger.exception("Error stopping infrastructure")
         return return_error(400, "Error stopping infrastructure: %s" % get_ex_error(ex))
@@ -983,7 +983,7 @@ def RESTStartVM(infid=None, vmid=None):
     except IncorrectVMException as ex:
         return return_error(404, "Error starting VM: %s" % get_ex_error(ex))
     except DisabledFunctionException as ex:
-        return return_error(403, "Error Destroying Inf: %s" % get_ex_error(ex))
+        return return_error(403, "Error starting VM: %s" % get_ex_error(ex))
     except Exception as ex:
         logger.exception("Error starting VM")
         return return_error(400, "Error starting VM: %s" % get_ex_error(ex))
@@ -1010,7 +1010,7 @@ def RESTStopVM(infid=None, vmid=None):
     except IncorrectVMException as ex:
         return return_error(404, "Error stopping VM: %s" % get_ex_error(ex))
     except DisabledFunctionException as ex:
-        return return_error(403, "Error Destroying Inf: %s" % get_ex_error(ex))
+        return return_error(403, "Error stopping VM: %s" % get_ex_error(ex))
     except Exception as ex:
         logger.exception("Error stopping VM")
         return return_error(400, "Error stopping VM: %s" % get_ex_error(ex))
@@ -1037,7 +1037,7 @@ def RESTRebootVM(infid=None, vmid=None):
     except IncorrectVMException as ex:
         return return_error(404, "Error rebooting VM: %s" % get_ex_error(ex))
     except DisabledFunctionException as ex:
-        return return_error(403, "Error Destroying Inf: %s" % get_ex_error(ex))
+        return return_error(403, "Error rebooting VM: %s" % get_ex_error(ex))
     except Exception as ex:
         logger.exception("Error rebooting VM")
         return return_error(400, "Error rebooting VM: %s" % get_ex_error(ex))
@@ -1049,7 +1049,7 @@ def ReturnOptions(**kwargs):
 
 
 @app.route('/version', method='GET')
-def RESTGeVersion():
+def RESTGetVersion():
     try:
         from IM import __version__ as version
         return format_output(version, field_name="version")
@@ -1094,7 +1094,7 @@ def RESTCreateDiskSnapshot(infid=None, vmid=None, disknum=None):
     except IncorrectVMException as ex:
         return return_error(404, "Error creating snapshot: %s" % get_ex_error(ex))
     except DisabledFunctionException as ex:
-        return return_error(403, "Error Destroying Inf: %s" % get_ex_error(ex))
+        return return_error(403, "Error creating snapshot: %s" % get_ex_error(ex))
     except Exception as ex:
         logger.exception("Error creating snapshot")
         return return_error(400, "Error creating snapshot: %s" % get_ex_error(ex))
@@ -1140,6 +1140,55 @@ def RESTGetCloudInfo(cloudid=None, param=None):
         return return_error(400, "Error getting cloud info: %s" % get_ex_error(ex))
 
 
+@app.route('/infrastructures/:infid/authorization', method='POST')
+def RESTChangeInfrastructureAuth(infid=None):
+    try:
+        auth = get_auth_header()
+    except Exception:
+        return return_error(401, "No authentication data provided")
+
+    try:
+        overwrite = False
+        if "overwrite" in bottle.request.params.keys():
+            str_overwrite = bottle.request.params.get("overwrite").lower()
+            if str_overwrite in ['yes', 'true', '1']:
+                overwrite = True
+            elif str_overwrite in ['no', 'false', '0']:
+                overwrite = False
+            else:
+                return return_error(400, "Incorrect value in overwrite parameter")
+
+        content_type = get_media_type('Content-Type') or ["application/json"]
+
+        if "application/json" in content_type:
+            auth_dict = json.loads(bottle.request.body.read().decode("utf-8"))
+            if "type" not in auth_dict:
+                auth_dict["type"] = "InfrastructureManager"
+            new_auth = Authentication([auth_dict])
+        else:
+            return return_error(415, "Unsupported Media Type %s" % content_type)
+
+        InfrastructureManager.ChangeInfrastructureAuth(infid, new_auth, overwrite, auth)
+        bottle.response.content_type = "text/plain"
+        return ""
+
+    except DeletedInfrastructureException as ex:
+        return return_error(404, "Error modifying infrastructure owner: %s" % get_ex_error(ex))
+    except IncorrectInfrastructureException as ex:
+        return return_error(404, "Error modifying infrastructure owners: %s" % get_ex_error(ex))
+    except UnauthorizedUserException as ex:
+        return return_error(403, "Error modifying infrastructure owner: %s" % get_ex_error(ex))
+    except DeletedVMException as ex:
+        return return_error(404, "Error modifying infrastructure owner: %s" % get_ex_error(ex))
+    except IncorrectVMException as ex:
+        return return_error(404, "Error modifying infrastructure owner: %s" % get_ex_error(ex))
+    except DisabledFunctionException as ex:
+        return return_error(403, "Error modifying infrastructure owner: %s" % get_ex_error(ex))
+    except Exception as ex:
+        logger.exception("Error modifying infrastructure owner.")
+        return return_error(400, "Error modifying infrastructure owner: %s" % get_ex_error(ex))
+
+
 @app.error(403)
 def error_mesage_403(error):
     return return_error(403, error.body)
