@@ -101,6 +101,8 @@ class Config:
     OIDC_CLIENT_ID = None
     OIDC_CLIENT_SECRET = None
     OIDC_SCOPES = []
+    OIDC_USER_INFO_PATH = "/userinfo"
+    OIDC_INSTROSPECT_PATH = "/introspect"
     VM_NUM_USE_CTXT_DIST = 30
     DELAY_BETWEEN_VM_RETRIES = 5
     VERIFI_SSL = False
