@@ -16,7 +16,7 @@
 
 import time
 import requests
-from netaddr import IPNetwork, spanning_cidr
+from netaddr import IPNetwork, IPAddress, spanning_cidr
 
 try:
     import boto.ec2
@@ -303,11 +303,11 @@ class EC2CloudConnector(CloudConnector):
                 if gpu and instace_type.gpu < gpu:
                     continue
 
-                if gpu_model and (not instace_type.gpu_model or
-                                  gpu_model.lower() not in instace_type.gpu_model.lower()):
+                if gpu and gpu_model and (not instace_type.gpu_model or
+                                          gpu_model.lower() not in instace_type.gpu_model.lower()):
                     continue
-                if gpu_vendor and (not instace_type.gpu_model or
-                                   gpu_vendor.lower() not in instace_type.gpu_model.lower()):
+                if gpu and gpu_vendor and (not instace_type.gpu_model or
+                                           gpu_vendor.lower() not in instace_type.gpu_model.lower()):
                     continue
 
                 if comparison:
@@ -394,30 +394,30 @@ class EC2CloudConnector(CloudConnector):
                 res.append(sg.id)
 
                 try:
-                    # open always SSH port
-                    sg.authorize('tcp', 22, 22, '0.0.0.0/0')
                     # open all the ports for the VMs in the security group
                     sg.authorize('tcp', 0, 65535, src_group=sg)
                     sg.authorize('udp', 0, 65535, src_group=sg)
                 except Exception as addex:
                     self.log_warn("Exception adding SG rules. Probably the rules exists:" + str(addex))
 
-                outports = network.getOutPorts()
-                if outports:
-                    for outport in outports:
-                        if outport.is_range():
-                            try:
-                                sg.authorize(outport.get_protocol(), outport.get_port_init(),
-                                             outport.get_port_end(), '0.0.0.0/0')
-                            except Exception as addex:
-                                self.log_warn("Exception adding SG rules. Probably the rules exists:" + str(addex))
-                        else:
-                            if outport.get_remote_port() != 22 or not network.isPublic():
-                                try:
-                                    sg.authorize(outport.get_protocol(), outport.get_remote_port(),
-                                                 outport.get_remote_port(), '0.0.0.0/0')
-                                except Exception as addex:
-                                    self.log_warn("Exception adding SG rules. Probably the rules exists:" + str(addex))
+                outports = network.getOutPorts() or []
+                # open always SSH port on public nets or private with proxy host
+                if network.isPublic() or network.getValue("proxy_host"):
+                    outports = self.add_ssh_port(outports)
+
+                for outport in outports:
+                    if outport.is_range():
+                        try:
+                            sg.authorize(outport.get_protocol(), outport.get_port_init(),
+                                         outport.get_port_end(), outport.get_remote_cidr())
+                        except Exception as addex:
+                            self.log_warn("Exception adding SG rules. Probably the rules exists:" + str(addex))
+                    else:
+                        try:
+                            sg.authorize(outport.get_protocol(), outport.get_remote_port(),
+                                         outport.get_remote_port(), outport.get_remote_cidr())
+                        except Exception as addex:
+                            self.log_warn("Exception adding SG rules. Probably the rules exists:" + str(addex))
 
                 i += 1
         except Exception as ex:
@@ -1031,7 +1031,13 @@ class EC2CloudConnector(CloudConnector):
             num_nets += 1
             num_pub_nets = 1
         if instance.private_ip_address is not None and len(instance.private_ip_address) > 0:
-            private_ips = [instance.private_ip_address]
+            is_private = any([IPAddress(instance.private_ip_address) in IPNetwork(mask)
+                              for mask in Config.PRIVATE_NET_MASKS])
+            if is_private:
+                private_ips = [instance.private_ip_address]
+            else:
+                public_ips = [instance.private_ip_address]
+                num_pub_nets += 1
             num_nets += 1
 
         vm.setIps(public_ips, private_ips)
@@ -1557,6 +1563,8 @@ class EC2CloudConnector(CloudConnector):
         """
         # Get info from http://www.ec2instances.info/
         # https://raw.githubusercontent.com/powdahound/ec2instances.info/master/www/instances.json
+        # It has been removed!!
+        # Copied in the IM Repo!.
 
         if EC2CloudConnector.instance_type_list:
             return EC2CloudConnector.instance_type_list
@@ -1566,7 +1574,7 @@ class EC2CloudConnector(CloudConnector):
         while cont < retries and not data:
             cont += 1
             try:
-                info_url = "https://raw.githubusercontent.com/powdahound/ec2instances.info/master/www/instances.json"
+                info_url = "https://raw.githubusercontent.com/grycap/im/devel/scripts/instances.json"
                 resp = requests.get(info_url)
                 if resp.status_code == 200:
                     data = resp.json()
@@ -1581,9 +1589,9 @@ class EC2CloudConnector(CloudConnector):
             self.log_error("Error getting ec2instances info.")
         else:
             for instance_type in data:
-                price = 50
-                if instance_type['pricing'] and 'us-east-1' in instance_type['pricing']:
-                    price = float(instance_type['pricing']['us-east-1']['linux']['ondemand'])
+                price = 200
+                if instance_type['pricing']:
+                    price = float(instance_type['pricing'])
                 disks = 0
                 disk_space = 0
                 if instance_type['storage']:
