@@ -18,6 +18,7 @@ import uuid
 import os
 import time
 import re
+from netaddr import IPNetwork, IPAddress
 
 try:
     from libcloud.compute.base import NodeSize, NodeState
@@ -39,6 +40,7 @@ except ImportError:
     from urllib.parse import urlparse
 from IM.VirtualMachine import VirtualMachine
 from radl.radl import Feature
+from IM.config import Config
 
 
 class GCECloudConnector(LibCloudCloudConnector):
@@ -83,13 +85,8 @@ class GCECloudConnector(LibCloudCloudConnector):
 
             if 'username' in auth and 'password' in auth and 'project' in auth:
                 cls = libcloud_get_driver(Provider.GCE)
-                # Patch to solve some client problems with \\n
+                # Patch to solve some client problems with \\n with certificates
                 auth['password'] = auth['password'].replace('\\n', '\n')
-                lines = len(auth['password'].replace(" ", "").split())
-                if lines < 2:
-                    raise Exception("The certificate provided to the GCE plugin has an incorrect format."
-                                    " Check that it has more than one line.")
-
                 driver = cls(auth['username'], auth['password'], project=auth['project'], datacenter=datacenter)
 
                 self.driver = driver
@@ -360,6 +357,7 @@ class GCECloudConnector(LibCloudCloudConnector):
             allowed = [{'IPProtocol': 'udp', 'ports': '1-65535'},
                        {'IPProtocol': 'tcp', 'ports': '1-65535'},
                        {'IPProtocol': 'icmp'}]
+            source_ranges = []
 
             try:
                 driver.ex_create_firewall(firewall_name, allowed, network=net_name, source_tags=['imid-%s' % inf.id])
@@ -376,23 +374,27 @@ class GCECloudConnector(LibCloudCloudConnector):
                 else:
                     firewall_name = "im-%s-%s" % (inf.id, net_name)
 
-                outports = public_net.getOutPorts()
-                if outports:
-                    for outport in outports:
-                        if outport.get_protocol() not in ports:
-                            ports[outport.get_protocol()] = []
-                        if outport.is_range():
-                            port_range = "%d-%d" % (outport.get_port_init(), outport.get_port_end())
-                            ports[outport.get_protocol()].append(port_range)
-                        elif outport.get_local_port() != 22:
-                            ports[outport.get_protocol()].append(str(outport.get_remote_port()))
+                outports = public_net.getOutPorts() or []
+                for outport in outports:
+                    if outport.get_remote_cidr() != "0.0.0.0/0":
+                        source_ranges.append(outport.get_remote_cidr())
+                    if outport.get_protocol() not in ports:
+                        ports[outport.get_protocol()] = []
+                    if outport.is_range():
+                        port_range = "%d-%d" % (outport.get_port_init(), outport.get_port_end())
+                        ports[outport.get_protocol()].append(port_range)
+                    elif outport.get_local_port() != 22:
+                        ports[outport.get_protocol()].append(str(outport.get_remote_port()))
 
                 allowed = [{'IPProtocol': 'tcp', 'ports': ports['tcp']}]
                 if 'udp' in ports:
                     allowed.append({'IPProtocol': 'udp', 'ports': ports['udp']})
 
+                if not source_ranges:
+                    source_ranges = ['0.0.0.0/0']
+
                 try:
-                    driver.ex_create_firewall(firewall_name, allowed, network=net_name)
+                    driver.ex_create_firewall(firewall_name, allowed, network=net_name, source_ranges=source_ranges)
                     self.log_info("Firewall %s successfully created." % firewall_name)
                 except ResourceExistsError:
                     self.log_debug("FW already exists. Ignore.")
@@ -585,8 +587,17 @@ class GCECloudConnector(LibCloudCloudConnector):
             if not net_provider_id:
                 net_provider_id = "default"
 
-        args['ex_network'] = net_provider_id
-        self.create_firewall(inf, net_provider_id, radl, driver)
+        if "." in net_provider_id:
+            parts = net_provider_id.split(".")
+            if len(parts) == 2:
+                args['ex_network'] = parts[0]
+                args['ex_subnetwork'] = parts[1]
+                self.create_firewall(inf, parts[0], radl, driver)
+            else:
+                raise Exception("Incorrect subnet value in provider_id value: %s" % net_provider_id)
+        else:
+            args['ex_network'] = net_provider_id
+            self.create_firewall(inf, net_provider_id, radl, driver)
 
         args['ex_can_ip_forward'] = self.can_ip_forward(radl)
 
@@ -853,7 +864,17 @@ class GCECloudConnector(LibCloudCloudConnector):
 
             self.update_system_info_from_instance(vm.info.systems[0], node.size)
 
-            vm.setIps(node.public_ips, node.private_ips)
+            private_ips = []
+            public_ips = list(node.public_ips)
+            # Use PRIVATE_NET_MASKS to set private IPs
+            for private_ip in node.private_ips:
+                is_private = any([IPAddress(private_ip) in IPNetwork(mask) for mask in Config.PRIVATE_NET_MASKS])
+                if is_private:
+                    private_ips.append(private_ip)
+                else:
+                    public_ips.append(private_ip)
+
+            vm.setIps(public_ips, private_ips)
             self.add_dns_entries(vm, auth_data)
             self.addRouterInstance(vm, driver)
         else:
