@@ -17,6 +17,8 @@
 import base64
 import json
 import requests
+import time
+from netaddr import IPNetwork, IPAddress
 try:
     from urlparse import urlparse
 except ImportError:
@@ -34,10 +36,6 @@ class KubernetesCloudConnector(CloudConnector):
 
     type = "Kubernetes"
 
-    _port_base_num = 35000
-    """ Base number to assign SSH port on Kubernetes node."""
-    _port_counter = 0
-    """ Counter to assign SSH port on Kubernetes node."""
     _root_password = "Aspecial+0ne"
     """ Default password to set to the root in the container"""
     _apiVersions = ["v1", "v1beta3"]
@@ -51,6 +49,14 @@ class KubernetesCloudConnector(CloudConnector):
     }
     """Dictionary with a map with the Kubernetes POD states to the IM states."""
 
+    def __init__(self, cloud_info, inf):
+        self.apiVersion = None
+        CloudConnector.__init__(self, cloud_info, inf)
+
+    def _get_api_url(self, auth_data, namespace, path):
+        apiVersion = self.get_api_version(auth_data)
+        return "/api/" + apiVersion + "/namespaces/" + namespace + path
+
     def create_request(self, method, url, auth_data, headers=None, body=None):
         auth_header = self.get_auth_header(auth_data)
         if auth_header:
@@ -58,8 +64,12 @@ class KubernetesCloudConnector(CloudConnector):
                 headers = {}
             headers.update(auth_header)
 
+        if body and isinstance(body, dict):
+            data = json.dumps(body)
+        else:
+            data = body
         url = "%s://%s:%d%s%s" % (self.cloud.protocol, self.cloud.server, self.cloud.port, self.cloud.path, url)
-        resp = requests.request(method, url, verify=self.verify_ssl, headers=headers, data=body)
+        resp = requests.request(method, url, verify=self.verify_ssl, headers=headers, data=data)
 
         return resp
 
@@ -70,9 +80,7 @@ class KubernetesCloudConnector(CloudConnector):
         url = urlparse(self.cloud.server)
         auths = auth_data.getAuthInfo(self.type, url[1])
         if not auths:
-            self.log_error(
-                "No correct auth data has been specified to Kubernetes.")
-            return None
+            raise Exception("No correct auth data has been specified to Kubernetes.")
         else:
             auth = auths[0]
 
@@ -86,6 +94,8 @@ class KubernetesCloudConnector(CloudConnector):
         elif 'token' in auth:
             token = auth['token']
             auth_header = {'Authorization': 'Bearer ' + token}
+        else:
+            raise Exception("No correct auth data has been specified to Kubernetes: username and password or token.")
 
         return auth_header
 
@@ -93,6 +103,9 @@ class KubernetesCloudConnector(CloudConnector):
         """
         Return the API version to use to connect with kubernetes API server
         """
+        if self.apiVersion:
+            return self.apiVersion
+
         version = self._apiVersions[0]
 
         try:
@@ -102,11 +115,11 @@ class KubernetesCloudConnector(CloudConnector):
                 output = json.loads(resp.text)
                 for v in self._apiVersions:
                     if v in output["versions"]:
+                        self.apiVersion = v
                         return v
 
         except Exception:
-            self.log_exception(
-                "Error connecting with Kubernetes API server")
+            self.log_exception("Error connecting with Kubernetes API server")
 
         self.log_warn("Error getting a compatible API version. Setting the default one.")
         self.log_debug("Using %s API version." % version)
@@ -131,9 +144,8 @@ class KubernetesCloudConnector(CloudConnector):
 
     def _delete_volume_claim(self, namespace, vc_name, auth_data):
         try:
-            apiVersion = self.get_api_version(auth_data)
-
-            uri = "/api/" + apiVersion + "/namespaces/" + namespace + "/persistentvolumeclaims/" + vc_name
+            self.log_debug("Deleting PVC: %s/%s" % (namespace, vc_name))
+            uri = self._get_api_url(auth_data, namespace, "/persistentvolumeclaims/" + vc_name)
             resp = self.create_request('DELETE', uri, auth_data)
 
             if resp.status_code == 404:
@@ -159,14 +171,9 @@ class KubernetesCloudConnector(CloudConnector):
 
     def _create_volume_claim(self, claim_data, auth_data):
         try:
-            apiVersion = self.get_api_version(auth_data)
-
             headers = {'Content-Type': 'application/json'}
-            uri = ("/api/" + apiVersion + "/namespaces/" +
-                   claim_data['metadata']['namespace'] +
-                   "/persistentvolumeclaims")
-            body = json.dumps(claim_data)
-            resp = self.create_request('POST', uri, auth_data, headers, body)
+            uri = self._get_api_url(auth_data, claim_data['metadata']['namespace'], "/persistentvolumeclaims")
+            resp = self.create_request('POST', uri, auth_data, headers, claim_data)
 
             output = str(resp.text)
             if resp.status_code != 201:
@@ -175,11 +182,10 @@ class KubernetesCloudConnector(CloudConnector):
             else:
                 return True
         except Exception:
-            self.log_exception(
-                "Error connecting with Kubernetes API server")
+            self.log_exception("Error connecting with Kubernetes API server")
             return False
 
-    def _create_volumes(self, apiVersion, namespace, system, pod_name, auth_data, persistent=False):
+    def _create_volumes(self, namespace, system, pod_name, auth_data, persistent=False):
         res = []
         cont = 1
         while (system.getValue("disk." + str(cont) + ".size") and
@@ -197,11 +203,12 @@ class KubernetesCloudConnector(CloudConnector):
             name = "%s-%d" % (pod_name, cont)
 
             if persistent:
-                claim_data = {'apiVersion': apiVersion, 'kind': 'PersistentVolumeClaim'}
+                claim_data = {'apiVersion': 'v1', 'kind': 'PersistentVolumeClaim'}
                 claim_data['metadata'] = {'name': name, 'namespace': namespace}
                 claim_data['spec'] = {'accessModes': ['ReadWriteOnce'], 'resources': {
                     'requests': {'storage': disk_size}}}
 
+                self.log_debug("Creating PVC: %s/%s" % (namespace, name))
                 success = self._create_volume_claim(claim_data, auth_data)
                 if success:
                     res.append((name, disk_device, disk_size, disk_mount_path, persistent))
@@ -214,46 +221,99 @@ class KubernetesCloudConnector(CloudConnector):
 
         return res
 
-    def _generate_pod_data(self, apiVersion, namespace, name, outports, system, ssh_port, volumes):
-        cpu = str(system.getValue('cpu.count'))
-        memory = "%s" % system.getFeature('memory.size').getValue('B')
-        # The URI has this format: docker://image_name
-        image_name = system.getValue("disk.0.image.url")[9:]
+    def create_service_data(self, namespace, name, outports, auth_data, vm):
+        try:
+            service_data = self._generate_service_data(namespace, name, outports)
+            self.log_debug("Creating Service: %s/%s" % (namespace, name))
+            headers = {'Content-Type': 'application/json'}
+            uri = self._get_api_url(auth_data, namespace, '/services')
+            svc_resp = self.create_request('POST', uri, auth_data, headers, service_data)
+            if svc_resp.status_code != 201:
+                self.error_messages += "Error creating service for pod %s: %s" % (name, svc_resp.text)
+                self.log_warn("Error creating service: %s" % svc_resp.text)
+            else:
+                # Wait a bit to assure the service has been created
+                time.sleep(0.5)
+                # Get Service data to get assigned nodePort
+                uri = self._get_api_url(auth_data, namespace, '/services/%s' % name)
+                svc_resp = self.create_request('GET', uri, auth_data)
+                if svc_resp.status_code == 200:
+                    for port in svc_resp.json()['spec']['ports']:
+                        # Set Out port in the RADL info of the VM
+                        vm.setOutPort(int(port['port']), int(port['nodePort']))
 
-        ports = [{'containerPort': 22, 'protocol': 'TCP', 'hostPort': ssh_port}]
+        except Exception:
+            self.error_messages += "Error creating service to access pod %s" % name
+            self.log_exception("Error creating service.")
+
+    def _generate_service_data(self, namespace, name, outports):
+        service_data = {'apiVersion': 'v1', 'kind': 'Service'}
+        service_data['metadata'] = {
+            'name': name,
+            'namespace': namespace,
+            'labels': {'name': name}
+        }
+
+        ports = [{'port': 22, 'targetPort': 22, 'protocol': 'TCP', 'name': 'ssh'}]
         if outports:
             for outport in outports:
                 if outport.is_range():
                     self.log_warn("Port range not allowed in Kubernetes connector. Ignoring.")
                 elif outport.get_local_port() != 22:
-                    ports.append({'containerPort': outport.get_local_port(), 'protocol': outport.get_protocol().upper(
-                    ), 'hostPort': outport.get_remote_port()})
+                    ports.append({'port': outport.get_local_port(), 'protocol': outport.get_protocol().upper(),
+                                  'targetPort': outport.get_local_port(), 'name': 'port%s' % outport.get_local_port()})
 
-        pod_data = {'apiVersion': apiVersion, 'kind': 'Pod'}
+        service_data['spec'] = {
+            'type': 'NodePort',
+            'ports': ports,
+            'selector': {'name': name}
+        }
+
+        return service_data
+
+    def _generate_pod_data(self, namespace, name, outports, system, volumes):
+        cpu = str(system.getValue('cpu.count'))
+        memory = "%s" % system.getFeature('memory.size').getValue('B')
+        # The URI has this format: docker://image_name
+        image_name = system.getValue("disk.0.image.url")[9:]
+
+        ports = [{'containerPort': 22, 'protocol': 'TCP'}]
+        if outports:
+            for outport in outports:
+                if outport.is_range():
+                    self.log_warn("Port range not allowed in Kubernetes connector. Ignoring.")
+                elif outport.get_local_port() != 22:
+                    ports.append({'containerPort': outport.get_local_port(),
+                                  'protocol': outport.get_protocol().upper()})
+
+        pod_data = {'apiVersion': 'v1', 'kind': 'Pod'}
         pod_data['metadata'] = {
             'name': name,
             'namespace': namespace,
             'labels': {'name': name}
         }
-        command = "yum install -y openssh-server python"
+        command = "yum install -y openssh-server python sudo"
         command += " ; "
-        command += "apt-get update && apt-get install -y openssh-server python"
+        command += "apt-get update && apt-get install -y openssh-server python sudo"
+        command += " ; "
+        command += "apk add --no-cache openssh-server python2 sudo"
         command += " ; "
         command += "mkdir /var/run/sshd"
         command += " ; "
-        command += "sed -i '/PermitRootLogin/c\PermitRootLogin yes' /etc/ssh/sshd_config"
+        command += "sed -i '/PermitRootLogin/c\\PermitRootLogin yes' /etc/ssh/sshd_config"
         command += " ; "
         command += "ssh-keygen -t rsa -f /etc/ssh/ssh_host_rsa_key -N ''"
         command += " ; "
         command += "echo 'root:" + self._root_password + "' | chpasswd"
         command += " ; "
-        command += "sed 's@session\s*required\s*pam_loginuid.so@session optional pam_loginuid.so@g' -i /etc/pam.d/sshd"
+        command += ("sed 's@session\\s*required\\s*pam_loginuid.so@session"
+                    " optional pam_loginuid.so@g' -i /etc/pam.d/sshd")
         command += " ; "
         command += " /usr/sbin/sshd -D"
         containers = [{
             'name': name,
             'image': image_name,
-            'command': ["/bin/bash", "-c", command],
+            'command': ["/bin/sh", "-c", command],
             'imagePullPolicy': 'IfNotPresent',
             'ports': ports,
             'resources': {'limits': {'cpu': cpu, 'memory': memory}}
@@ -299,26 +359,31 @@ class KubernetesCloudConnector(CloudConnector):
         if public_net:
             outports = public_net.getOutPorts()
 
-        apiVersion = self.get_api_version(auth_data)
-
         res = []
         # First create the namespace for the infrastructure
         namespace = inf.id
         headers = {'Content-Type': 'application/json'}
-        uri = "/api/" + apiVersion + "/namespaces"
+        uri = self._get_api_url(auth_data, "", "")
         with inf._lock:
-            resp = self.create_request('GET', uri + "/" + namespace, auth_data, headers)
+            resp = self.create_request('GET', uri + namespace, auth_data, headers)
             if resp.status_code != 200:
-                namespace_data = {'apiVersion': apiVersion, 'kind': 'Namespace',
+                self.log_debug("Creating Namespace: %s" % namespace)
+                namespace_data = {'apiVersion': 'v1', 'kind': 'Namespace',
                                   'metadata': {'name': namespace}}
-                body = json.dumps(namespace_data)
-                resp = self.create_request('POST', uri, auth_data, headers, body)
+                resp = self.create_request('POST', uri, auth_data, headers, namespace_data)
 
                 if resp.status_code != 201:
                     for _ in range(num_vm):
                         res.append((False, "Error creating the Namespace: " + resp.text))
                         return res
 
+                # we need to assure it has been created before creating other resources
+                resp = self.create_request('GET', uri + namespace, auth_data, headers)
+                if resp.status_code != 200:
+                    for _ in range(num_vm):
+                        res.append((False, "Error creating the Namespace"))
+                        return res
+
         i = 0
         while i < num_vm:
             try:
@@ -332,33 +397,29 @@ class KubernetesCloudConnector(CloudConnector):
                 pod_name = nodename
 
                 # Do not use the Persistent volumes yet
-                volumes = self._create_volumes(apiVersion, namespace, system, pod_name, auth_data)
+                volumes = self._create_volumes(namespace, system, pod_name, auth_data)
 
-                ssh_port = (KubernetesCloudConnector._port_base_num + KubernetesCloudConnector._port_counter) % 65535
-                KubernetesCloudConnector._port_counter += 1
-                pod_data = self._generate_pod_data(apiVersion, namespace, pod_name, outports,
-                                                   system, ssh_port, volumes)
-                body = json.dumps(pod_data)
+                pod_data = self._generate_pod_data(namespace, pod_name, outports, system, volumes)
 
-                uri = "/api/" + apiVersion + "/namespaces/" + namespace + "/pods"
-                resp = self.create_request('POST', uri, auth_data, headers, body)
+                self.log_debug("Creating POD: %s/%s" % (namespace, pod_name))
+                uri = self._get_api_url(auth_data, namespace, '/pods')
+                resp = self.create_request('POST', uri, auth_data, headers, pod_data)
 
                 if resp.status_code != 201:
+                    self.log_error("Error creating the Container: " + resp.text)
                     res.append((False, "Error creating the Container: " + resp.text))
                     try:
                         self._delete_volume_claims(pod_data, auth_data)
                     except Exception:
                         self.log_exception("Error deleting volumes.")
                 else:
+                    self.create_service_data(namespace, pod_name, outports, auth_data, vm)
+
                     output = json.loads(resp.text)
                     vm.id = output["metadata"]["name"]
-                    # Set SSH port in the RADL info of the VM
-                    vm.setSSHPort(ssh_port)
                     # Set the default user and password to access the container
-                    vm.info.systems[0].setValue(
-                        'disk.0.os.credentials.username', 'root')
-                    vm.info.systems[0].setValue(
-                        'disk.0.os.credentials.password', self._root_password)
+                    vm.info.systems[0].setValue('disk.0.os.credentials.username', 'root')
+                    vm.info.systems[0].setValue('disk.0.os.credentials.password', self._root_password)
                     vm.info.systems[0].setValue('instance_id', str(vm.id))
                     vm.info.systems[0].setValue('instance_name', str(vm.id))
 
@@ -376,9 +437,7 @@ class KubernetesCloudConnector(CloudConnector):
             namespace = vm.inf.id
             pod_name = vm.id
 
-            apiVersion = self.get_api_version(auth_data)
-
-            uri = "/api/" + apiVersion + "/namespaces/" + namespace + "/pods/" + pod_name
+            uri = self._get_api_url(auth_data, namespace, "/pods/" + pod_name)
             resp = self.create_request('GET', uri, auth_data)
 
             if resp.status_code == 200:
@@ -387,16 +446,14 @@ class KubernetesCloudConnector(CloudConnector):
                 return (False, resp.status_code, resp.text)
 
         except Exception as ex:
-            self.log_exception(
-                "Error connecting with Kubernetes API server")
+            self.log_exception("Error connecting with Kubernetes API server")
             return (False, None, "Error connecting with Kubernetes API server: " + str(ex))
 
     def updateVMInfo(self, vm, auth_data):
         success, status, output = self._get_pod(vm, auth_data)
         if success:
             output = json.loads(output)
-            vm.state = self.VM_STATE_MAP.get(
-                output["status"]["phase"], VirtualMachine.UNKNOWN)
+            vm.state = self.VM_STATE_MAP.get(output["status"]["phase"], VirtualMachine.UNKNOWN)
 
             # Update the network info
             self.setIPs(vm, output)
@@ -405,8 +462,7 @@ class KubernetesCloudConnector(CloudConnector):
             self.log_error("Error getting info about the POD: code: %s, msg: %s" % (status, output))
             return (False, "Error getting info about the POD: code: %s, msg: %s" % (status, output))
 
-    @staticmethod
-    def setIPs(vm, pod_info):
+    def setIPs(self, vm, pod_info):
         """
         Adapt the RADL information of the VM to the real IPs assigned by the cloud provider
 
@@ -418,24 +474,33 @@ class KubernetesCloudConnector(CloudConnector):
         public_ips = []
         private_ips = []
         if 'hostIP' in pod_info["status"]:
-            public_ips = [str(pod_info["status"]["hostIP"])]
+            host_ip = str(pod_info["status"]["hostIP"])
+            is_private = any([IPAddress(host_ip) in IPNetwork(mask) for mask in Config.PRIVATE_NET_MASKS])
+            if is_private:
+                public_ips = [self.cloud.server]
+            else:
+                public_ips = [host_ip]
         if 'podIP' in pod_info["status"]:
             private_ips = [str(pod_info["status"]["podIP"])]
 
         vm.setIps(public_ips, private_ips)
 
     def finalize(self, vm, last, auth_data):
+        msg = ""
         if vm.id:
             success, status, output = self._get_pod(vm, auth_data)
             if success:
                 if status == 404:
                     self.log_warn("Trying to remove a non existing POD id: %s" % vm.id)
-                    return (True, vm.id)
                 else:
                     pod_data = json.loads(output)
                     self._delete_volume_claims(pod_data, auth_data)
+                    success, msg = self._delete_pod(vm, auth_data)
+                    if not success:
+                        self.log_error("Error deleting Pod %s: %s" % (vm.id, msg))
+                        return False, "Error deleting Pod %s: %s" % (vm.id, msg)
 
-            success = self._delete_pod(vm, auth_data)
+            success, msg = self._delete_service(vm, auth_data)
         else:
             self.log_warn("No VM ID. Ignoring")
             success = True
@@ -443,25 +508,46 @@ class KubernetesCloudConnector(CloudConnector):
         if last:
             self._delete_namespace(vm, auth_data)
 
-        return success
+        return success, msg
 
     def _delete_namespace(self, vm, auth_data):
-        apiVersion = self.get_api_version(auth_data)
-        headers = {'Content-Type': 'application/json'}
-        uri = "/api/" + apiVersion + "/namespaces/" + vm.inf.id
-        resp = self.create_request('DELETE', uri, auth_data, headers)
-        if resp.status_code != 200:
+        self.log_debug("Deleting Namespace: %s" % vm.inf.id)
+        uri = self._get_api_url(auth_data, vm.inf.id, '')
+        resp = self.create_request('DELETE', uri, auth_data)
+        if resp.status_code == 404:
+            self.log_warn("Trying to remove a non existing Namespace id: " + vm.inf.id)
+        elif resp.status_code != 200:
             self.log_error("Error deleting Namespace")
             return False
         return True
 
+    def _delete_service(self, vm, auth_data):
+        try:
+            namespace = vm.inf.id
+            service_name = vm.id
+
+            self.log_debug("Deleting Service: %s/%s" % (namespace, service_name))
+            uri = self._get_api_url(auth_data, namespace, "/services/" + service_name)
+            resp = self.create_request('DELETE', uri, auth_data)
+
+            if resp.status_code == 404:
+                self.log_warn("Trying to remove a non existing Service id: " + service_name)
+                return (True, service_name)
+            elif resp.status_code != 200:
+                return (False, "Error deleting the Service: " + resp.text)
+            else:
+                return (True, service_name)
+        except Exception:
+            self.log_exception("Error connecting with Kubernetes API server")
+            return (False, "Error connecting with Kubernetes API server")
+
     def _delete_pod(self, vm, auth_data):
         try:
             namespace = vm.inf.id
             pod_name = vm.id
 
-            apiVersion = self.get_api_version(auth_data)
-            uri = "/api/" + apiVersion + "/namespaces/" + namespace + "/pods/" + pod_name
+            self.log_debug("Deleting POD: %s/%s" % (namespace, pod_name))
+            uri = self._get_api_url(auth_data, namespace, "/pods/" + pod_name)
             resp = self.create_request('DELETE', uri, auth_data)
 
             if resp.status_code == 404:
@@ -489,8 +575,6 @@ class KubernetesCloudConnector(CloudConnector):
         # But kubernetes does not permit cpu to be updated yet
         system = radl.systems[0]
 
-        apiVersion = self.get_api_version(auth_data)
-
         try:
             pod_data = []
 
@@ -519,9 +603,8 @@ class KubernetesCloudConnector(CloudConnector):
             pod_name = vm.id
 
             headers = {'Content-Type': 'application/json-patch+json'}
-            uri = "/api/" + apiVersion + "/namespaces/" + namespace + "/pods/" + pod_name
-            body = json.dumps(pod_data)
-            resp = self.create_request('PATCH', uri, auth_data, headers, body)
+            uri = self._get_api_url(auth_data, namespace, "/pods/" + pod_name)
+            resp = self.create_request('PATCH', uri, auth_data, headers, pod_data)
 
             if resp.status_code != 201:
                 return (False, "Error updating the Pod: " + resp.text)
