@@ -314,9 +314,6 @@ class OpenNebulaCloudConnector(CloudConnector):
             if disk.TARGET and not vm.info.systems[0].getValue("disk." + str(disk.DISK_ID) + ".device"):
                 vm.info.systems[0].setValue(
                     "disk." + str(disk.DISK_ID) + ".device", disk.TARGET)
-            if disk.FORMAT and not vm.info.systems[0].getValue("disk." + str(disk.DISK_ID) + ".fstype"):
-                vm.info.systems[0].setValue(
-                    "disk." + str(disk.DISK_ID) + ".fstype", disk.FORMAT)
 
     @staticmethod
     def setIPsFromTemplate(vm, template):
@@ -570,7 +567,11 @@ class OpenNebulaCloudConnector(CloudConnector):
             self.delete_snapshots(vm, auth_data)
 
         if vm.id:
-            success, err = server.one.vm.action(session_id, 'delete', int(vm.id))[0:2]
+            one_ver = self.getONEVersion(auth_data)
+            op = 'terminate'
+            if one_ver <= LooseVersion("4.14.0"):
+                op = 'delete'
+            success, err = server.one.vm.action(session_id, op, int(vm.id))[0:2]
         else:
             self.log_warn("No VM ID. Ignoring")
             err = ""
@@ -645,11 +646,8 @@ class OpenNebulaCloudConnector(CloudConnector):
             else:
                 disk_size = system.getFeature(
                     "disk." + str(cont) + ".size").getValue('M')
-                disk_fstype = system.getValue("disk." + str(cont) + ".fstype")
-                if not disk_fstype:
-                    disk_fstype = 'ext3'
 
-                disks += ' DISK = [ SAVE = no, TYPE = fs , FORMAT = %s, SIZE = %d' % (disk_fstype, int(disk_size))
+                disks += ' DISK = [ SAVE = no, TYPE = fs , FORMAT = qcow2, SIZE = %d' % int(disk_size)
 
             if disk_device:
                 disks += ', TARGET = %s' % disk_device
@@ -1176,7 +1174,7 @@ class OpenNebulaCloudConnector(CloudConnector):
 
         image_type = ""  # Use the default one
         one_ver = self.getONEVersion(auth_data)
-        if one_ver.startswith("5."):
+        if one_ver >= LooseVersion("5.0"):
             success, res_info = server.one.vm.disksaveas(session_id, int(vm.id), disk_num,
                                                          image_name, image_type, -1)[0:2]
         else:
@@ -1330,15 +1328,16 @@ class OpenNebulaCloudConnector(CloudConnector):
                "instances": {"used": 0, "limit": -1},
                "security_groups": {"used": 0, "limit": -1}}
 
-        res["cores"]["used"] = user_info.VM_QUOTA.VM.CPU_USED
-        if user_info.VM_QUOTA.VM.CPU >= 0:
-            res["cores"]["limit"] = user_info.VM_QUOTA.VM.CPU
-        res["ram"]["used"] = user_info.VM_QUOTA.VM.MEMORY_USED / 1024
-        if user_info.VM_QUOTA.VM.MEMORY >= 0:
-            res["ram"]["limit"] = user_info.VM_QUOTA.VM.MEMORY / 1024
-        res["instances"]["used"] = user_info.VM_QUOTA.VM.VMS_USED
-        if user_info.VM_QUOTA.VM.VMS >= 0:
-            res["instances"]["limit"] = user_info.VM_QUOTA.VM.VMS
+        if user_info.VM_QUOTA and user_info.VM_QUOTA.VM:
+            res["cores"]["used"] = user_info.VM_QUOTA.VM.CPU_USED
+            if user_info.VM_QUOTA.VM.CPU >= 0:
+                res["cores"]["limit"] = user_info.VM_QUOTA.VM.CPU
+            res["ram"]["used"] = user_info.VM_QUOTA.VM.MEMORY_USED / 1024
+            if user_info.VM_QUOTA.VM.MEMORY >= 0:
+                res["ram"]["limit"] = user_info.VM_QUOTA.VM.MEMORY / 1024
+            res["instances"]["used"] = user_info.VM_QUOTA.VM.VMS_USED
+            if user_info.VM_QUOTA.VM.VMS >= 0:
+                res["instances"]["limit"] = user_info.VM_QUOTA.VM.VMS
 
         # Ine ONE map floating IPs to public IP leases
         one_nets = self.getONENetworks(auth_data)
