@@ -21,13 +21,17 @@ import os.path
 import tempfile
 import requests
 import base64
+from IM.CloudInfo import CloudInfo
 
 try:
     from libcloud.common.exceptions import BaseHTTPError
     from libcloud.compute.types import Provider
     from libcloud.compute.providers import get_driver
     from libcloud.compute.base import NodeAuthSSHKey
-    from libcloud.compute.drivers.openstack import OpenStack_2_NodeDriver, OpenStack_2_SubNet, OpenStackSecurityGroup
+    from libcloud.compute.drivers.openstack import (OpenStack_2_NodeDriver,
+                                                    OpenStack_2_SubNet,
+                                                    OpenStackSecurityGroup,
+                                                    OpenStackNodeSize)
 except Exception as ex:
     print("WARN: libcloud library not correctly installed. OpenStackCloudConnector will not work!.")
     print(ex)
@@ -59,6 +63,8 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
     """ Enable config drive """
     CONFIRM_TIMEOUT = 120
     """ Confirm Timeout """
+    DEFAULT_AUTH_VERSION = '2.0_password'
+    """ Default authentication method """
 
     def __init__(self, cloud_info, inf):
         self.auth = None
@@ -76,8 +82,50 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
         """
         driver = self.get_driver(auth_data)
         node = driver.ex_get_node_details(node_id)
+        # for old infras add cloud extra fields
+        if not self.cloud.extra:
+            CloudInfo.add_extra_fields(auth_data.getAuthInfo(self.type, self.cloud.server)[0], self.cloud)
         return node
 
+    def get_auth(self, auths):
+        """
+        Get a compatible auth data.
+        The auth must have the same auth_version and other
+        field equal depeding on the auth_version used.
+
+        Arguments:
+                - auth(Authentication): parsed authentication tokens.
+
+        Returns: a :py:class:`IM.auth.Authentication`
+        """
+        # in OLD infras extra field is not set
+        if not self.cloud.extra:
+            # Use the old behaviour
+            return auths[0]
+
+        if 'auth_version' not in self.cloud.extra or not self.cloud.extra['auth_version']:
+            self.cloud.extra['auth_version'] = self.DEFAULT_AUTH_VERSION
+        for auth in auths:
+            valid = True
+            fields = ['auth_version']
+            if self.cloud.extra['auth_version'] == "3.x_oidc_access_token":
+                fields.extend(['username', 'domain'])
+            elif "password" in self.cloud.extra['auth_version']:
+                fields.append('tenant')
+
+            if 'auth_version' not in auth or not auth['auth_version']:
+                auth['auth_version'] = self.DEFAULT_AUTH_VERSION
+
+            for field in fields:
+                if auth.get(field) != self.cloud.extra.get(field):
+                    valid = False
+                    break
+
+            if valid:
+                return auth
+
+        raise Exception("No compatible OpenStack auth data has been specified.")
+
     def get_driver(self, auth_data):
         """
         Get the driver from the auth data
@@ -91,26 +139,17 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
         if not auths:
             raise Exception("No auth data has been specified to OpenStack.")
         else:
-            auth = auths[0]
+            auth = self.get_auth(auths)
 
         if self.driver and self.auth.compare(auth_data, self.type, self.cloud.server):
             return self.driver
         else:
             self.auth = auth_data
 
-            protocol = self.cloud.protocol
-            if not protocol:
-                protocol = "http"
-            port = self.cloud.port
-            if port == -1:
-                if protocol == "http":
-                    port = 80
-                elif protocol == "https":
-                    port = 443
-                else:
-                    raise Exception("Invalid port/protocol specified for OpenStack site: %s" % self.cloud.server)
+            protocol = self.cloud.protocol or "http"
+            port = self.cloud.get_port()
 
-            parameters = {"auth_version": '2.0_password',
+            parameters = {"auth_version": self.DEFAULT_AUTH_VERSION,
                           "auth_url": protocol + "://" + self.cloud.server + ":" + str(port),
                           "auth_token": None,
                           "service_type": None,
@@ -304,7 +343,7 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
         src_host = url[1].split(':')[0]
 
         if protocol == "appdb":
-            site_url, image_id, msg = AppDB.get_image_data(str_url, "openstack")
+            site_url, image_id, msg = AppDB.get_image_data(str_url, "openstack", site=self.cloud.server)
             if not image_id or not site_url:
                 self.log_error(msg)
                 return None
@@ -482,13 +521,24 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
 
             try:
                 flavorId = node.extra['flavorId']
-                instance_type = node.driver.ex_get_size(flavorId)
-                if len(instance_type.extra) == 0:
-                    try:
-                        # get it now
-                        instance_type.extra = node.driver.ex_get_size_extra_specs(instance_type.id)
-                    except Exception:
-                        self.log_exception("Error trying to get flavor '%s' extra_specs." % instance_type.id)
+                if flavorId:
+                    instance_type = node.driver.ex_get_size(flavorId)
+                    if len(instance_type.extra) == 0:
+                        try:
+                            # get it now
+                            instance_type.extra = node.driver.ex_get_size_extra_specs(instance_type.id)
+                        except Exception:
+                            self.log_exception("Error trying to get flavor '%s' extra_specs." % instance_type.id)
+                elif node.extra['flavor_details']:
+                    fdetails = node.extra['flavor_details']
+                    instance_type = OpenStackNodeSize("id", fdetails.get("original_name"),
+                                                      fdetails.get("ram"),
+                                                      fdetails.get("disk"),
+                                                      None, None, node.driver,
+                                                      fdetails.get("vcpus"),
+                                                      fdetails.get("ephemeral"),
+                                                      fdetails.get("swap"),
+                                                      fdetails.get("extra", {}))
                 self.update_system_info_from_instance(vm.info.systems[0], instance_type)
             except Exception as ex:
                 self.log_warn("Error updating VM info from flavor ID: %s" % get_ex_error(ex))
@@ -614,6 +664,15 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
 
         return res
 
+    def get_floating_ip_node_id(self, ip):
+        """Get floating ip node."""
+        try:
+            # for OpenStack_2_FloatingIpAddress
+            return ip.get_node_id()
+        except Exception:
+            # for OpenStack_1_1_FloatingIpAddress
+            return ip.node_id
+
     def get_node_floating_ips(self, node):
         """
         Get a list of ip addresses associated with a node
@@ -622,7 +681,7 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
         try:
             for pool in node.driver.ex_list_floating_ip_pools():
                 for ip in pool.list_floating_ips():
-                    if ip.node_id == node.id:
+                    if self.get_floating_ip_node_id(ip) == node.id:
                         ips.append(ip.ip_address)
         except BaseHTTPError as ex:
             if ex.code == 404:
@@ -703,7 +762,7 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
                             priv_ips.append(ipu)
                         else:
                             pub_ips.append(ipu)
-                    vm.setIps(pub_ips, priv_ips)
+                    vm.setIps(pub_ips, priv_ips, False)
 
         else:
             # if addresses are not available use the old method
@@ -714,7 +773,7 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
                     private_ips.append(ip)
                 else:
                     public_ips.append(ip)
-            vm.setIps(public_ips, private_ips, True)
+            vm.setIps(public_ips, private_ips)
 
         if vm.state == VirtualMachine.RUNNING:
             if self.add_public_ip_count < self.MAX_ADD_IP_COUNT:
@@ -1172,7 +1231,7 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
         image_url = system.getValue("disk.0.image.url")
         if urlparse(image_url)[0] == "appdb":
             vo = self.get_vo_name(auth_data)
-            _, image_id, msg = AppDB.get_image_data(image_url, "openstack", vo)
+            _, image_id, msg = AppDB.get_image_data(image_url, "openstack", vo, site=self.cloud.server)
             if not image_id:
                 self.log_error(msg)
                 raise Exception("Error in appdb image: %s" % msg)
@@ -1208,6 +1267,9 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
                 'ex_security_groups': sgs,
                 'name': self.gen_instance_name(system)}
 
+        if system.getValue('availability_zone'):
+            args['ex_availability_zone'] = system.getValue('availability_zone')
+
         if blockdevicemappings:
             args['ex_blockdevicemappings'] = blockdevicemappings
 
@@ -1349,7 +1411,7 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
         Get a floating IP
         """
         for ip in pool.list_floating_ips():
-            if not ip.node_id:
+            if not self.get_floating_ip_node_id(ip):
                 is_private = any([IPAddress(ip.ip_address) in IPNetwork(mask) for mask in Config.PRIVATE_NET_MASKS])
                 if is_private:
                     self.log_info("Floating IP found %s, but it is private. Ignore." % ip.ip_address)
@@ -1496,35 +1558,38 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
                 res.append(sg)
 
             try:
-                # open always SSH port on public nets or private with proxy host
-                if network.isPublic() or network.getValue("proxy_host"):
-                    driver.ex_create_security_group_rule(sg, 'tcp', 22, 22, '0.0.0.0/0')
                 # open all the ports for the VMs in the security group
                 driver.ex_create_security_group_rule(sg, 'tcp', 1, 65535, source_security_group=sg)
                 driver.ex_create_security_group_rule(sg, 'udp', 1, 65535, source_security_group=sg)
             except Exception as addex:
                 self.log_warn("Exception adding SG rules. Probably the rules exists: %s" % get_ex_error(addex))
 
-            outports = network.getOutPorts()
-            if outports:
-                for outport in outports:
-                    if outport.is_range():
-                        try:
-                            driver.ex_create_security_group_rule(sg, outport.get_protocol(),
-                                                                 outport.get_port_init(),
-                                                                 outport.get_port_end(), '0.0.0.0/0')
-                        except Exception as ex:
-                            self.log_warn("Exception adding SG rules: %s" % get_ex_error(ex))
-                    else:
-                        if outport.get_remote_port() != 22 or not network.isPublic():
-                            try:
-                                driver.ex_create_security_group_rule(sg, outport.get_protocol(),
-                                                                     outport.get_remote_port(),
-                                                                     outport.get_remote_port(), '0.0.0.0/0')
-                            except Exception as ex:
-                                self.log_warn("Exception adding SG rules: %s" % get_ex_error(ex))
-                                self.error_messages += ("Exception adding port %s to SG rules.\n" %
-                                                        outport.get_remote_port())
+            outports = network.getOutPorts() or []
+            # open always SSH port on public nets or private with proxy host
+            if network.isPublic() or network.getValue("proxy_host"):
+                outports = self.add_ssh_port(outports)
+
+            for outport in outports:
+                if outport.is_range():
+                    try:
+                        driver.ex_create_security_group_rule(sg, outport.get_protocol(),
+                                                             outport.get_port_init(),
+                                                             outport.get_port_end(),
+                                                             outport.get_remote_cidr())
+                    except Exception as ex:
+                        self.log_warn("Exception adding SG rules: %s" % get_ex_error(ex))
+                        self.error_messages += ("Exception adding port range: %s-%s to SG rules.\n" %
+                                                (outport.get_port_init(), outport.get_port_end()))
+                else:
+                    try:
+                        driver.ex_create_security_group_rule(sg, outport.get_protocol(),
+                                                             outport.get_remote_port(),
+                                                             outport.get_remote_port(),
+                                                             outport.get_remote_cidr())
+                    except Exception as ex:
+                        self.log_warn("Exception adding SG rules: %s" % get_ex_error(ex))
+                        self.error_messages += ("Exception adding port %s to SG rules.\n" %
+                                                outport.get_remote_port())
 
         return res
 
@@ -1906,7 +1971,7 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
                 no_delete_ips = vm.floating_ips
 
             for floating_ip in node.driver.ex_list_floating_ips():
-                if floating_ip.node_id == node.id:
+                if self.get_floating_ip_node_id(floating_ip) == node.id:
                     # remove it from the node
                     try:
                         node.driver.ex_detach_floating_ip_from_node(node, floating_ip)
