@@ -4,6 +4,8 @@ import yaml
 import copy
 import operator
 import requests
+import json
+import re
 from toscaparser.nodetemplate import NodeTemplate
 
 try:
@@ -133,6 +135,44 @@ class Tosca:
                         else:
                             Tosca.logger.warn("Unknown tyoe of token %s. Ignoring." % token_type)
                 radl.ansible_hosts = [ansible_host]
+            elif root_type == "tosca.nodes.aisprint.FaaS.Function":
+                oscar_host = self._find_host_compute(node, self.tosca.nodetemplates,
+                                                     "tosca.nodes.SoftwareComponent")
+                oscar_compute = self._find_host_compute(node, self.tosca.nodetemplates)
+                # If the function has a host create a recipe
+                if oscar_compute and oscar_host:
+                    service_json = self._get_oscar_service_json(node)
+                    dns_host = self._final_function_result(oscar_host.get_property_value('dns_host'), oscar_host)
+                    if dns_host and dns_host.strip("'\""):
+                        service_endpoint = "https://%s" % dns_host
+                    else:
+                        service_endpoint = "http://{{ IM_NODE_PUBLIC_IP }}"
+                    service_password = self._final_function_result(oscar_host.get_property_value('password'),
+                                                                   oscar_host)
+
+                    recipe = '  - tasks:\n'
+                    recipe += '    - include_tasks: utils/tasks/oscar_function.yml\n'
+                    recipe += '      vars:\n'
+                    recipe += '        oscar_endpoint: "%s"\n' % service_endpoint
+                    recipe += '        oscar_username: "oscar"\n'
+                    recipe += '        oscar_password: "%s"\n' % service_password
+                    recipe += "        oscar_service_json: '%s'\n" % service_json
+
+                    conf = configure("oscar_%s" % node.name, recipe)
+                    radl.configures.append(conf)
+                    level = Tosca._get_dependency_level(node)
+                    cont_items.append(contextualize_item(oscar_compute.name, conf.name, level))
+                else:
+                    # if not create a system
+                    oscar_sys = self._gen_oscar_system(node)
+                    radl.systems.append(oscar_sys)
+                    conf = configure(node.name, None)
+                    radl.configures.append(conf)
+                    level = Tosca._get_dependency_level(node)
+                    cont_items.append(contextualize_item(node.name, conf.name, level))
+                    cloud_id = self._get_placement_property(oscar_sys.name, "cloud_id")
+                    dep = deploy(oscar_sys.name, 1, cloud_id)
+                    radl.deploys.append(dep)
             else:
                 if root_type == "tosca.nodes.Compute":
                     # Add the system RADL element
@@ -204,6 +244,9 @@ class Tosca:
         Check private networks to assure to create different nets
         for different cloud providers
         """
+        if not radl.networks:
+            return
+
         priv_net_cloud_map = {}
 
         # in case of an AddResource
@@ -304,8 +347,11 @@ class Tosca:
         for port in ports_dict.values():
             protocol = "tcp"
             source_range = None
+            remote_cidr = ""
             if "protocol" in port:
                 protocol = port["protocol"]
+            if "remote_cidr" in port:
+                remote_cidr = "%s-" % port["remote_cidr"]
             if "source_range" in port:
                 source_range = port["source_range"]
             else:
@@ -322,11 +368,11 @@ class Tosca:
             if source_range:
                 if res:
                     res += ","
-                res += "%s:%s/%s" % (source_range[0], source_range[1], protocol)
+                res += "%s%s:%s/%s" % (remote_cidr, source_range[0], source_range[1], protocol)
             else:
                 if res:
                     res += ","
-                res += "%s/%s-%s/%s" % (remote_port, protocol, local_port, protocol)
+                res += "%s%s/%s-%s/%s" % (remote_cidr, remote_port, protocol, local_port, protocol)
 
         return res
 
@@ -383,6 +429,7 @@ class Tosca:
                     # This is for the special case of OST with net name and pool name
                     network_name = parts[0].strip()
                     pool_name = parts[1].strip()
+                    public_ip = True
 
                 if network_name == "PUBLIC":
                     public_ip = True
@@ -412,6 +459,10 @@ class Tosca:
                 if "protocol" in cap_props:
                     protocol = self._final_function_result(cap_props["protocol"].value, node)
                 ports["im-%s-%s" % (protocol, port)] = {"protocol": protocol, "source": port}
+                if "remote_cidr" in cap_props:
+                    remote_cidr = self._final_function_result(cap_props["remote_cidr"].value, node)
+                    if remote_cidr:
+                        ports["im-%s-%s" % (protocol, port)]['remote_cidr'] = remote_cidr
             if cap_props and "additional_ip" in cap_props:
                 additional_ip = self._final_function_result(cap_props["additional_ip"].value, node)
 
@@ -893,6 +944,78 @@ class Tosca:
                     "Intrinsic function %s not supported." % func_name)
                 return None
 
+    def _get_default_attribute(self, node, attribute_name, inf_info=None):
+        """Get the default value set to an attribute."""
+        try:
+            node_type = node.type_definition
+        except AttributeError:
+            node_type = node.definition
+
+        attributes_def = node_type.get_attributes_def()
+        if attribute_name in attributes_def:
+            attr_schema = attributes_def[attribute_name].schema
+            if 'default' in attr_schema:
+                return self._final_function_result(attr_schema['default'], node, inf_info)
+
+        return None
+
+    def _get_ansible_output(self, cont_out, attribute_params):
+        """
+        Get values from ansible output.
+
+        Accepted parameters: ctxt_task_name, ansible_task_name
+        Each ansible task has the following attributes:
+        - state: str with the ansible state returned
+        - output: str only available in some tasks (as debug ones)
+        """
+        if not cont_out:
+            return None
+
+        ctxt_task = {}
+        for task in cont_out.split("Launch task: ")[1:]:
+            task_name = task[:task.find("\n")]
+            ctxt_task[task_name] = {'tasks': {}}
+            ansible_task_name = None
+
+            for num, atask in enumerate(task.split("\nTASK [")[1:]):
+                ansible_task_name = atask[:atask.find("] **")]
+                if ansible_task_name in ctxt_task[task_name]:
+                    ansible_task_name = "%s_%d" % (ansible_task_name, num)
+                ctxt_task[task_name]['tasks'][ansible_task_name] = {'state': None, 'output': None}
+
+                state_search = re.search("\n([a-z]+): \[.*\]", atask)
+                if state_search:
+                    ctxt_task[task_name]['tasks'][ansible_task_name]['state'] = state_search.group(1)
+                output_search = re.search("\n[a-z]+: \[.*\] => \{", atask)
+                if output_search:
+                    output_endpos = atask.find("}\n", output_search.end(0))
+                    output = atask[output_search.end(0) - 1:output_endpos + 1]
+                    try:
+                        json_out = json.loads(output)
+                        if "var" in json_out:
+                            json_out = json_out["var"]
+                        if len(json_out) == 1:
+                            output = json[list(json_out.keys())[0]]
+                        if "msg" in json_out:
+                            output = json_out["msg"]
+                        ctxt_task[task_name]['tasks'][ansible_task_name]['output'] = output
+                    except Exception:
+                        ctxt_task[task_name]['tasks'][ansible_task_name]['output'] = output
+
+        return self._get_object_values(ctxt_task, attribute_params)
+
+    @staticmethod
+    def _get_object_values(value, attribute_params):
+        """Get values from dict."""
+        res = value
+        for param in attribute_params:
+            if param in res:
+                res = res[param]
+            else:
+                Tosca.logger.error("Invalid map index: %s in value: %s" % (param, value))
+                return None
+        return res
+
     def _get_attribute_result(self, func, node, inf_info):
         """Get an attribute value of an entity defined in the service template
 
@@ -900,6 +1023,10 @@ class Tosca:
         responsibility of the Tosca engine to implement the evaluation of
         get_attribute functions.
 
+        Grammar:
+        get_attribute: [ <modelable_entity_name>, <optional_req_or_cap_name>, <attribute_name>,
+                         <nested_attribute_name_or_index_1>, ..., <nested_attribute_name_or_index_n> ]
+
         Arguments:
 
         * Node template name | HOST.
@@ -918,26 +1045,12 @@ class Tosca:
         * { get_attribute: [ HOST, private_address, 0 ] }
         * { get_attribute: [ server, endpoint, credential, 0 ] }
         """
+        if len(func.args) < 2:
+            Tosca.logger.error("Calling get_attribute function. Min 2 parameters.")
+            return None
         node_name = func.args[0]
-        capability_name = None
-        attribute_name = func.args[1]
-
-        index = None
-        # Currently only support 2,3 or 4 parameters
-        if len(func.args) == 3:
-            try:
-                index = int(func.args[2])
-            except Exception:
-                capability_name = func.args[1]
-                attribute_name = func.args[2]
-        elif len(func.args) == 4:
-            capability_name = func.args[1]
-            attribute_name = func.args[2]
-            try:
-                index = int(func.args[3])
-            except Exception:
-                Tosca.logger.exception("Error getting get_attribute index.")
 
+        # Get node
         if node_name == "HOST":
             node = self._find_host_compute(node, self.tosca.nodetemplates)
         elif node_name == "SOURCE":
@@ -956,17 +1069,50 @@ class Tosca:
             return None
 
         #  if capability_name refers a requirement, try to get the referred node
+        capability_name = func.args[1]
+        args_cont = 1
+        cap_or_req_names = list(node.get_capabilities().keys())
+        cap_or_req_names.extend([name for r in node.requirements for name, _ in r.items()])
+        if capability_name in cap_or_req_names:
+            args_cont += 1
+            if len(func.args) < 3:
+                Tosca.logger.error("Calling get_attribute function. Min 3 parameters with cap or req name.")
+                return None
+        else:
+            capability_name = None
+            Tosca.logger.debug("%s is not a cap or req on node %s. It must be an attribute." % (capability_name,
+                                                                                                node.name))
+
+        # Find capability in node template's requirements
         if capability_name:
-            # Find attribute in node template's requirements
             for r in node.requirements:
                 for req, name in r.items():
                     if req == capability_name:
                         node = func._find_node_template(name)
 
-        host_node = self._find_host_compute(node, self.tosca.nodetemplates)
+        attribute_name = func.args[args_cont]
+        attribute_params = func.args[args_cont + 1:]
+
+        # Index is the value of the first int parameter
+        index = None
+        if attribute_params:
+            try:
+                index = int(attribute_params[0])
+                attribute_params.pop(0)
+            except Exception:
+                Tosca.logger.debug("First parameter is not an index.")
+
+        attribute_default = self._get_default_attribute(node, attribute_name, inf_info)
+        if attribute_default:
+            return attribute_default
 
         root_type = Tosca._get_root_parent_type(node).type
 
+        host_node = self._find_host_compute(node, self.tosca.nodetemplates)
+        if root_type == "tosca.nodes.aisprint.FaaS.Function" and host_node is None:
+            # in case of FaaS functions without host, the node is the host
+            host_node = node
+
         if inf_info:
             vm_list = inf_info.get_vm_list_by_system_name()
 
@@ -990,6 +1136,13 @@ class Tosca:
                     Tosca.logger.warn("Attribute ctxt_log only supported"
                                       " in tosca.nodes.indigo.Compute nodes.")
                     return None
+            elif attribute_name == "ansible_output":
+                if node.type == "tosca.nodes.indigo.Compute":
+                    return self._get_ansible_output(vm.cont_out, attribute_params)
+                else:
+                    Tosca.logger.warn("Attribute ansible_output only supported"
+                                      " in tosca.nodes.indigo.Compute nodes.")
+                    return None
             elif attribute_name == "credential" and capability_name == "endpoint":
                 if node.type == "tosca.nodes.indigo.Compute":
                     res = []
@@ -1002,7 +1155,7 @@ class Tosca:
                         if private_key:
                             val["token_type"] = "private_key"
                             val["token"] = private_key
-                        res.append(val)
+                        res.append(self._get_object_values(val, attribute_params))
                     if index is not None:
                         res = res[index]
                     return res
@@ -1035,6 +1188,64 @@ class Tosca:
                         return vm.getPublicIP()
                     else:
                         return vm.getPrivateIP()
+            elif attribute_name == "endpoint":
+                if root_type == "tosca.nodes.aisprint.FaaS.Function":
+                    oscar_host = self._find_host_compute(node, self.tosca.nodetemplates,
+                                                         "tosca.nodes.SoftwareComponent")
+                    if host_node != node and oscar_host:
+                        # OSCAR function deployed in a deployed VM
+                        dns_host = self._final_function_result(oscar_host.get_property_value('dns_host'), oscar_host)
+                        if dns_host and dns_host.strip("'\""):
+                            return "https://%s" % dns_host
+
+                        if vm.getPublicIP():
+                            return "http://%s" % vm.getPublicIP()
+                        else:
+                            return "http://%s" % vm.getPrivateIP()
+                    else:
+                        # OSCAR function deployed in a pre-deployed cluster or not dns_host set
+                        return vm.getCloudConnector().cloud.get_url()
+
+                Tosca.logger.warn("Attribute endpoint only supported in tosca.nodes.aisprint.FaaS.Function")
+                return None
+            elif attribute_name == "credential":
+                if root_type == "tosca.nodes.aisprint.FaaS.Function":
+                    oscar_host = self._find_host_compute(node, self.tosca.nodetemplates,
+                                                         "tosca.nodes.SoftwareComponent")
+                    if host_node != node and oscar_host:
+                        # OSCAR function deployed in a deployed VM
+                        oscar_pass = self._final_function_result(oscar_host.get_property_value('password'), oscar_host)
+                        if oscar_pass and oscar_pass.strip("'\""):
+                            return self._get_object_values({"user": "oscar",
+                                                            "token_type": "password",
+                                                            "token": oscar_pass},
+                                                           attribute_params)
+
+                        Tosca.logger.warn("No password defined in tosca.nodes.indigo.OSCAR host node")
+                        return None
+                    else:
+                        # OSCAR function deployed in a pre-deployed cluster or not dns_host set
+                        if vm.getCloudConnector().auth:
+                            auth = vm.getCloudConnector().auth
+                            if 'username' in auth and 'password' in auth:
+                                return self._get_object_values({"user": auth["username"],
+                                                                "token_type": "password",
+                                                                "token": auth["password"]},
+                                                               attribute_params)
+                            elif 'token' in auth:
+                                return self._get_object_values({"user": "",
+                                                                "token_type": "bearer",
+                                                                "token": auth["token"]},
+                                                               attribute_params)
+                            else:
+                                Tosca.logger.warn("No valid auth data in OSCAR connector")
+                                return None
+
+                        Tosca.logger.warn("No auth data in OSCAR connector")
+                        return None
+
+                Tosca.logger.warn("Attribute credential only supported in tosca.nodes.aisprint.FaaS.Function")
+                return None
             else:
                 Tosca.logger.warn("Attribute %s not supported." % attribute_name)
                 return None
@@ -1084,6 +1295,18 @@ class Tosca:
                                 "hostvars[groups['%s'][0]]['IM_NODE_PRIVATE_IP']}}" % (host_node.name,
                                                                                        host_node.name,
                                                                                        host_node.name))
+            elif attribute_name == "endpoint":
+                if root_type == "tosca.nodes.aisprint.FaaS.Function":
+                    oscar_host = self._find_host_compute(node, self.tosca.nodetemplates,
+                                                         "tosca.nodes.SoftwareComponent")
+                    if host_node != node and oscar_host:
+                        # OSCAR function deployed in a deployed VM
+                        dns_host = self._final_function_result(oscar_host.get_property_value('dns_host'), oscar_host)
+                        if dns_host.strip("'\""):
+                            return "https://%s" % dns_host
+
+                Tosca.logger.warn("Attribute endpoint only supported in tosca.nodes.aisprint.FaaS.Function")
+                return None
             else:
                 Tosca.logger.warn("Attribute %s not supported." % attribute_name)
                 return None
@@ -1123,7 +1346,7 @@ class Tosca:
         # TODO: resolve function values related with run-time values as IM
         # or ansible variables
 
-    def _find_host_compute(self, node, nodetemplates):
+    def _find_host_compute(self, node, nodetemplates, base_root_type="tosca.nodes.Compute"):
         """
         Select the node to host each node, using the node requirements
         In most of the cases the are directly specified, otherwise "node_filter" is used
@@ -1131,21 +1354,21 @@ class Tosca:
 
         # check for a HosteOn relation
         root_type = Tosca._get_root_parent_type(node).type
-        if root_type == "tosca.nodes.Compute":
+        if root_type == base_root_type:
             return node
 
         if node.requirements:
             for r, n in node.relationships.items():
                 if Tosca._is_derived_from(r, r.HOSTEDON) or Tosca._is_derived_from(r, r.BINDSTO):
                     root_type = Tosca._get_root_parent_type(n).type
-                    if root_type == "tosca.nodes.Compute":
+                    if root_type == base_root_type:
                         return n
                     else:
                         return self._find_host_compute(n, nodetemplates)
 
         # There are no direct HostedOn node
         # check node_filter requirements
-        if node.requirements:
+        if node.requirements and base_root_type == "tosca.nodes.Compute":
             for requires in node.requirements:
                 if 'host' in requires:
                     value = requires.get('host')
@@ -1661,3 +1884,94 @@ class Tosca:
             yaml1 = yaml2
 
         return yaml1
+
+    def _gen_oscar_system(self, node):
+        """Generate the system for an OSCAR function."""
+        res = system(node.name)
+
+        property_map = {
+            'name': 'name',
+            'cpu': 'cpu.count',
+            'image': 'disk.0.image.url',
+            'script': 'script',
+            'image_pull_secrets': 'image_pull_secrets'
+        }
+
+        for prop in node.get_properties_objects():
+            value = self._final_function_result(prop.value, node)
+            if value not in [None, [], {}]:
+                if prop.name in property_map:
+                    res.setValue(property_map[prop.name], value)
+                elif prop.name == 'alpine':
+                    res.setValue('alpine', 1 if value else 0)
+                elif prop.name == 'memory':
+                    if not value.endswith("B"):
+                        value += "B"
+                    value = int(ScalarUnit_Size(value).get_num_from_scalar_unit('B'))
+                    res.setValue("memory.size", value, 'B')
+                elif prop.name == 'env_variables':
+                    variables = ["%s:%s" % (k, v) for k, v in value.items()]
+                    res.setValue("environment.variables", variables)
+                elif prop.name in ['input', 'output']:
+                    for num, elem in enumerate(value):
+                        res.setValue("%s.%d.provider" % (prop.name, num), elem.get("storage_provider"))
+                        res.setValue("%s.%d.path" % (prop.name, num), elem.get("path"))
+                        if elem.get("suffix"):
+                            res.setValue("%s.%d.suffix" % (prop.name, num), elem.get("suffix"))
+                        if elem.get("prefix"):
+                            res.setValue("%s.%d.prefix" % (prop.name, num), elem.get("prefix"))
+                elif prop.name == 'storage_providers':
+                    cont = {"minio": 0, "s3": 0, "onedata": 0}
+
+                    for provider_type in ["minio", "s3", "onedata"]:
+                        if provider_type in value:
+                            for prov_id, provider in value[provider_type].items():
+                                provider_pref = "%s.%d" % (provider_type, cont[provider_type])
+                                cont[provider_type] += 1
+                                res.setValue("%s.id" % provider_pref, prov_id)
+
+                                for elem in ['access_key', 'secret_key', 'region', 'endpoint',
+                                             'verify', 'oneprovider_host', 'token', 'space']:
+                                    if provider.get(elem):
+                                        res.setValue("%s.%s" % (provider_pref, elem), provider.get(elem))
+                else:
+                    # this should never happen
+                    Tosca.logger.warn("Property %s not expected. Ignoring." % prop.name)
+
+        return res
+
+    def _get_oscar_service_json(self, node):
+        """Get the OSCAR service json"""
+        res = {}
+
+        for prop in node.get_properties_objects():
+            value = self._final_function_result(prop.value, node)
+            if value not in [None, [], {}]:
+                if prop.name in ['name', 'script', 'alpine', 'input', 'output', 'storage_providers']:
+                    res[prop.name] = value
+                elif prop.name == 'cpu':
+                    res['cpu'] = "%g" % value
+                elif prop.name == 'memory':
+                    if not value.endswith("B"):
+                        value += "B"
+                    res[prop.name] = "%gMi" % ScalarUnit_Size(value).get_num_from_scalar_unit('MiB')
+                elif prop.name == 'image':
+                    if value.startswith('oscar://'):
+                        url_image = urlparse(value)
+                        if url_image.path:
+                            res["image"] = url_image.path[1:]
+                    elif value.startswith('docker://'):
+                        res["image"] = value[9:]
+                    else:
+                        res["image"] = value
+                elif prop.name == 'env_variables':
+                    res['environment'] = {'Variables': value}
+                elif prop.name == 'image_pull_secrets':
+                    if not isinstance(value, list):
+                        value = [value]
+                    res['image_pull_secrets'] = value
+                else:
+                    # this should never happen
+                    Tosca.logger.warn("Property %s not expected. Ignoring." % prop.name)
+
+        return json.dumps(res)
